@@ -29,16 +29,6 @@ const POLICIES_TESTS = [
    * },
    */
 
-  // POLICY: RememberPasswords
-  {
-    policies: { OfferToSaveLogins: false },
-    lockedPrefs: { "signon.rememberSignons": false },
-  },
-  {
-    policies: { OfferToSaveLogins: true },
-    lockedPrefs: { "signon.rememberSignons": true },
-  },
-
   // POLICY: DisableSecurityBypass
   {
     policies: {
@@ -95,12 +85,25 @@ const POLICIES_TESTS = [
       EnableTrackingProtection: {
         Cryptomining: true,
         Fingerprinting: true,
+        EmailTracking: true,
         Locked: true,
       },
     },
     lockedPrefs: {
       "privacy.trackingprotection.cryptomining.enabled": true,
       "privacy.trackingprotection.fingerprinting.enabled": true,
+      "privacy.trackingprotection.emailtracking.enabled": true,
+      "privacy.trackingprotection.emailtracking.pbmode.enabled": true,
+    },
+  },
+
+  // POLICY: GoToIntranetSiteForSingleWordEntryInAddressBar
+  {
+    policies: {
+      GoToIntranetSiteForSingleWordEntryInAddressBar: true,
+    },
+    lockedPrefs: {
+      "browser.fixup.dns_first_for_single_words": true,
     },
   },
 
@@ -224,6 +227,31 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.sessions": true,
       "privacy.clearOnShutdown.siteSettings": true,
       "privacy.clearOnShutdown.offlineApps": true,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": true,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": true,
+      "privacy.clearOnShutdown_v2.cache": true,
+      "privacy.clearOnShutdown_v2.siteSettings": true,
+    },
+  },
+
+  {
+    policies: {
+      SanitizeOnShutdown: false,
+    },
+    lockedPrefs: {
+      "privacy.sanitize.sanitizeOnShutdown": false,
+      "privacy.clearOnShutdown.cache": false,
+      "privacy.clearOnShutdown.cookies": false,
+      "privacy.clearOnShutdown.downloads": false,
+      "privacy.clearOnShutdown.formdata": false,
+      "privacy.clearOnShutdown.history": false,
+      "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown.siteSettings": false,
+      "privacy.clearOnShutdown.offlineApps": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
+      "privacy.clearOnShutdown_v2.siteSettings": false,
     },
   },
 
@@ -241,6 +269,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": true,
     },
   },
 
@@ -258,6 +289,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": true,
+      "privacy.clearOnShutdown_v2.cache": false,
     },
   },
 
@@ -275,6 +309,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
     },
   },
 
@@ -292,6 +329,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": true,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
     },
   },
 
@@ -309,6 +349,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": true,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": true,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
     },
   },
 
@@ -326,6 +369,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": true,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
     },
   },
 
@@ -344,6 +390,10 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
       "privacy.clearOnShutdown.siteSettings": true,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
+      "privacy.clearOnShutdown_v2.siteSettings": true,
     },
   },
 
@@ -362,6 +412,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
       "privacy.clearOnShutdown.offlineApps": true,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": false,
     },
   },
 
@@ -376,6 +429,7 @@ const POLICIES_TESTS = [
     lockedPrefs: {
       "privacy.sanitize.sanitizeOnShutdown": true,
       "privacy.clearOnShutdown.cache": true,
+      "privacy.clearOnShutdown_v2.cache": true,
     },
     unlockedPrefs: {
       "privacy.clearOnShutdown.cookies": false,
@@ -383,6 +437,8 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
     },
   },
 
@@ -398,12 +454,15 @@ const POLICIES_TESTS = [
       "privacy.sanitize.sanitizeOnShutdown": true,
       "privacy.clearOnShutdown.cache": true,
       "privacy.clearOnShutdown.cookies": false,
+      "privacy.clearOnShutdown_v2.cache": true,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
     },
     unlockedPrefs: {
       "privacy.clearOnShutdown.downloads": false,
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
     },
   },
 
@@ -422,23 +481,9 @@ const POLICIES_TESTS = [
       "privacy.clearOnShutdown.formdata": false,
       "privacy.clearOnShutdown.history": false,
       "privacy.clearOnShutdown.sessions": false,
-    },
-  },
-
-  // POLICY: DNSOverHTTPS Locked
-  {
-    policies: {
-      DNSOverHTTPS: {
-        Enabled: true,
-        ProviderURL: "http://example.com/provider",
-        ExcludedDomains: ["example.com", "example.org"],
-        Locked: true,
-      },
-    },
-    lockedPrefs: {
-      "network.trr.mode": 2,
-      "network.trr.uri": "http://example.com/provider",
-      "network.trr.excluded-domains": "example.com,example.org",
+      "privacy.clearOnShutdown_v2.historyFormDataAndDownloads": false,
+      "privacy.clearOnShutdown_v2.cookiesAndStorage": false,
+      "privacy.clearOnShutdown_v2.cache": true,
     },
   },
 
@@ -447,13 +492,43 @@ const POLICIES_TESTS = [
     policies: {
       DNSOverHTTPS: {
         Enabled: false,
-        ProviderURL: "http://example.com/provider",
+        ProviderURL: "https://example.com/provider",
         ExcludedDomains: ["example.com", "example.org"],
       },
     },
     unlockedPrefs: {
       "network.trr.mode": 5,
-      "network.trr.uri": "http://example.com/provider",
+      "network.trr.uri": "https://example.com/provider",
+      "network.trr.excluded-domains": "example.com,example.org",
+    },
+  },
+
+  // POLICY: DNSOverHTTPS Fallback off
+  {
+    policies: {
+      DNSOverHTTPS: {
+        Enabled: true,
+        Fallback: false,
+      },
+    },
+    unlockedPrefs: {
+      "network.trr.mode": 3,
+    },
+  },
+
+  // POLICY: DNSOverHTTPS Locked
+  {
+    policies: {
+      DNSOverHTTPS: {
+        Enabled: true,
+        ProviderURL: "https://example.com/provider",
+        ExcludedDomains: ["example.com", "example.org"],
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.trr.mode": 2,
+      "network.trr.uri": "https://example.com/provider",
       "network.trr.excluded-domains": "example.com,example.org",
     },
   },
@@ -550,12 +625,10 @@ const POLICIES_TESTS = [
     policies: {
       FirefoxHome: {
         Pocket: false,
-        Snippets: false,
         Locked: true,
       },
     },
     lockedPrefs: {
-      "browser.newtabpage.activity-stream.feeds.snippets": false,
       "browser.newtabpage.activity-stream.feeds.system.topstories": false,
     },
   },
@@ -570,17 +643,25 @@ const POLICIES_TESTS = [
     },
   },
 
+  // POLICY: RememberPasswords
+  {
+    policies: { OfferToSaveLogins: false },
+    lockedPrefs: { "signon.rememberSignons": false },
+  },
+  {
+    policies: { OfferToSaveLogins: true },
+    lockedPrefs: { "signon.rememberSignons": true },
+  },
+
   // POLICY: UserMessaging
   {
     policies: {
       UserMessaging: {
-        WhatsNew: false,
         SkipOnboarding: true,
         Locked: true,
       },
     },
     lockedPrefs: {
-      "browser.messaging-system.whatsNewPanel.enabled": false,
       "browser.aboutwelcome.enabled": false,
     },
   },
@@ -590,6 +671,7 @@ const POLICIES_TESTS = [
     policies: {
       UserMessaging: {
         SkipOnboarding: false,
+        Locked: false,
       },
     },
     unlockedPrefs: {
@@ -601,6 +683,7 @@ const POLICIES_TESTS = [
     policies: {
       UserMessaging: {
         ExtensionRecommendations: false,
+        Locked: false,
       },
     },
     unlockedPrefs: {
@@ -612,6 +695,7 @@ const POLICIES_TESTS = [
     policies: {
       UserMessaging: {
         FeatureRecommendations: false,
+        Locked: false,
       },
     },
     unlockedPrefs: {
@@ -620,6 +704,19 @@ const POLICIES_TESTS = [
   },
 
   // POLICY: Permissions->Autoplay
+  {
+    policies: {
+      Permissions: {
+        Autoplay: {
+          Default: "block-audio-video",
+        },
+      },
+    },
+    unlockedPrefs: {
+      "media.autoplay.default": 5,
+    },
+  },
+
   {
     policies: {
       Permissions: {
@@ -639,6 +736,7 @@ const POLICIES_TESTS = [
       Permissions: {
         Autoplay: {
           Default: "block-audio",
+          Locked: false,
         },
       },
     },
@@ -647,19 +745,6 @@ const POLICIES_TESTS = [
     },
   },
 
-  {
-    policies: {
-      Permissions: {
-        Autoplay: {
-          Default: "block-audio-video",
-        },
-      },
-    },
-    unlockedPrefs: {
-      "media.autoplay.default": 5,
-    },
-  },
-
   // POLICY: LegacySameSiteCookieBehaviorEnabled
 
   {
@@ -706,15 +791,39 @@ const POLICIES_TESTS = [
     policies: {
       PDFjs: {
         Enabled: false,
-        EnablePermissions: true,
       },
     },
     lockedPrefs: {
       "pdfjs.disabled": true,
+    },
+  },
+
+  {
+    policies: {
+      PDFjs: {
+        Enabled: true,
+        EnablePermissions: true,
+      },
+    },
+    lockedPrefs: {
+      "pdfjs.disabled": false,
       "pdfjs.enablePermissions": true,
     },
   },
 
+  {
+    policies: {
+      PDFjs: {
+        Enabled: true,
+        EnablePermissions: false,
+      },
+    },
+    lockedPrefs: {
+      "pdfjs.disabled": false,
+      "pdfjs.enablePermissions": false,
+    },
+  },
+
   // POLICY: PictureInPicture
 
   {
@@ -733,62 +842,98 @@ const POLICIES_TESTS = [
   {
     policies: {
       DisabledCiphers: {
-        TLS_DHE_RSA_WITH_AES_128_CBC_SHA: false,
-        TLS_DHE_RSA_WITH_AES_256_CBC_SHA: false,
-        TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA: false,
-        TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA: false,
         TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256: false,
         TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256: false,
+        TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256: false,
+        TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256: false,
+        TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384: false,
+        TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384: false,
+        TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA: false,
+        TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA: false,
+        TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA: false,
+        TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA: false,
+        TLS_DHE_RSA_WITH_AES_128_CBC_SHA: false,
+        TLS_DHE_RSA_WITH_AES_256_CBC_SHA: false,
+        TLS_RSA_WITH_AES_128_GCM_SHA256: false,
+        TLS_RSA_WITH_AES_256_GCM_SHA384: false,
         TLS_RSA_WITH_AES_128_CBC_SHA: false,
         TLS_RSA_WITH_AES_256_CBC_SHA: false,
         TLS_RSA_WITH_3DES_EDE_CBC_SHA: false,
-        TLS_RSA_WITH_AES_128_GCM_SHA256: false,
-        TLS_RSA_WITH_AES_256_GCM_SHA384: false,
+        TLS_CHACHA20_POLY1305_SHA256: false,
+        TLS_AES_128_GCM_SHA256: false,
+        TLS_AES_256_GCM_SHA384: false,
       },
     },
     lockedPrefs: {
-      "security.ssl3.dhe_rsa_aes_128_sha": true,
-      "security.ssl3.dhe_rsa_aes_256_sha": true,
-      "security.ssl3.ecdhe_rsa_aes_128_sha": true,
-      "security.ssl3.ecdhe_rsa_aes_256_sha": true,
       "security.ssl3.ecdhe_rsa_aes_128_gcm_sha256": true,
       "security.ssl3.ecdhe_ecdsa_aes_128_gcm_sha256": true,
-      "security.ssl3.rsa_aes_128_sha": true,
-      "security.ssl3.rsa_aes_256_sha": true,
-      "security.ssl3.rsa_des_ede3_sha": true,
+      "security.ssl3.ecdhe_ecdsa_chacha20_poly1305_sha256": true,
+      "security.ssl3.ecdhe_rsa_chacha20_poly1305_sha256": true,
+      "security.ssl3.ecdhe_ecdsa_aes_256_gcm_sha384": true,
+      "security.ssl3.ecdhe_rsa_aes_256_gcm_sha384": true,
+      "security.ssl3.ecdhe_rsa_aes_128_sha": true,
+      "security.ssl3.ecdhe_ecdsa_aes_128_sha": true,
+      "security.ssl3.ecdhe_rsa_aes_256_sha": true,
+      "security.ssl3.ecdhe_ecdsa_aes_256_sha": true,
+      "security.ssl3.dhe_rsa_aes_128_sha": true,
+      "security.ssl3.dhe_rsa_aes_256_sha": true,
       "security.ssl3.rsa_aes_128_gcm_sha256": true,
       "security.ssl3.rsa_aes_256_gcm_sha384": true,
+      "security.ssl3.rsa_aes_128_sha": true,
+      "security.ssl3.rsa_aes_256_sha": true,
+      "security.ssl3.deprecated.rsa_des_ede3_sha": true,
+      "security.tls13.chacha20_poly1305_sha256": true,
+      "security.tls13.aes_128_gcm_sha256": true,
+      "security.tls13.aes_256_gcm_sha384": true,
     },
   },
 
   {
     policies: {
       DisabledCiphers: {
-        TLS_DHE_RSA_WITH_AES_128_CBC_SHA: true,
-        TLS_DHE_RSA_WITH_AES_256_CBC_SHA: true,
-        TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA: true,
-        TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA: true,
         TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256: true,
         TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256: true,
+        TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256: true,
+        TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256: true,
+        TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384: true,
+        TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384: true,
+        TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA: true,
+        TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA: true,
+        TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA: true,
+        TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA: true,
+        TLS_DHE_RSA_WITH_AES_128_CBC_SHA: true,
+        TLS_DHE_RSA_WITH_AES_256_CBC_SHA: true,
+        TLS_RSA_WITH_AES_128_GCM_SHA256: true,
+        TLS_RSA_WITH_AES_256_GCM_SHA384: true,
         TLS_RSA_WITH_AES_128_CBC_SHA: true,
         TLS_RSA_WITH_AES_256_CBC_SHA: true,
         TLS_RSA_WITH_3DES_EDE_CBC_SHA: true,
-        TLS_RSA_WITH_AES_128_GCM_SHA256: true,
-        TLS_RSA_WITH_AES_256_GCM_SHA384: true,
+        TLS_CHACHA20_POLY1305_SHA256: true,
+        TLS_AES_128_GCM_SHA256: true,
+        TLS_AES_256_GCM_SHA384: true,
       },
     },
     lockedPrefs: {
-      "security.ssl3.dhe_rsa_aes_128_sha": false,
-      "security.ssl3.dhe_rsa_aes_256_sha": false,
-      "security.ssl3.ecdhe_rsa_aes_128_sha": false,
-      "security.ssl3.ecdhe_rsa_aes_256_sha": false,
       "security.ssl3.ecdhe_rsa_aes_128_gcm_sha256": false,
       "security.ssl3.ecdhe_ecdsa_aes_128_gcm_sha256": false,
-      "security.ssl3.rsa_aes_128_sha": false,
-      "security.ssl3.rsa_aes_256_sha": false,
-      "security.ssl3.rsa_des_ede3_sha": false,
+      "security.ssl3.ecdhe_ecdsa_chacha20_poly1305_sha256": false,
+      "security.ssl3.ecdhe_rsa_chacha20_poly1305_sha256": false,
+      "security.ssl3.ecdhe_ecdsa_aes_256_gcm_sha384": false,
+      "security.ssl3.ecdhe_rsa_aes_256_gcm_sha384": false,
+      "security.ssl3.ecdhe_rsa_aes_128_sha": false,
+      "security.ssl3.ecdhe_ecdsa_aes_128_sha": false,
+      "security.ssl3.ecdhe_rsa_aes_256_sha": false,
+      "security.ssl3.ecdhe_ecdsa_aes_256_sha": false,
+      "security.ssl3.dhe_rsa_aes_128_sha": false,
+      "security.ssl3.dhe_rsa_aes_256_sha": false,
       "security.ssl3.rsa_aes_128_gcm_sha256": false,
       "security.ssl3.rsa_aes_256_gcm_sha384": false,
+      "security.ssl3.rsa_aes_128_sha": false,
+      "security.ssl3.rsa_aes_256_sha": false,
+      "security.ssl3.deprecated.rsa_des_ede3_sha": false,
+      "security.tls13.chacha20_poly1305_sha256": false,
+      "security.tls13.aes_128_gcm_sha256": false,
+      "security.tls13.aes_256_gcm_sha384": false,
     },
   },
 
@@ -800,6 +945,220 @@ const POLICIES_TESTS = [
       "network.http.windows-sso.enabled": true,
     },
   },
+
+  {
+    policies: {
+      Cookies: {
+        Behavior: "accept",
+        BehaviorPrivateBrowsing: "reject-foreign",
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.cookie.cookieBehavior": 0,
+      "network.cookie.cookieBehavior.pbmode": 1,
+    },
+  },
+
+  {
+    policies: {
+      Cookies: {
+        Behavior: "reject-foreign",
+        BehaviorPrivateBrowsing: "reject",
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.cookie.cookieBehavior": 1,
+      "network.cookie.cookieBehavior.pbmode": 2,
+    },
+  },
+
+  {
+    policies: {
+      Cookies: {
+        Behavior: "reject",
+        BehaviorPrivateBrowsing: "limit-foreign",
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.cookie.cookieBehavior": 2,
+      "network.cookie.cookieBehavior.pbmode": 3,
+    },
+  },
+
+  {
+    policies: {
+      Cookies: {
+        Behavior: "limit-foreign",
+        BehaviorPrivateBrowsing: "reject-tracker",
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.cookie.cookieBehavior": 3,
+      "network.cookie.cookieBehavior.pbmode": 4,
+    },
+  },
+
+  {
+    policies: {
+      Cookies: {
+        Behavior: "reject-tracker",
+        BehaviorPrivateBrowsing: "reject-tracker-and-partition-foreign",
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.cookie.cookieBehavior": 4,
+      "network.cookie.cookieBehavior.pbmode": 5,
+    },
+  },
+  {
+    policies: {
+      Cookies: {
+        Behavior: "reject-tracker-and-partition-foreign",
+        BehaviorPrivateBrowsing: "accept",
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.cookie.cookieBehavior": 5,
+      "network.cookie.cookieBehavior.pbmode": 0,
+    },
+  },
+
+  {
+    policies: {
+      UseSystemPrintDialog: true,
+    },
+    lockedPrefs: {
+      "print.prefer_system_dialog": true,
+    },
+  },
+
+  // Bug 1820195
+  {
+    policies: {
+      Preferences: {
+        "pdfjs.cursorToolOnLoad": {
+          Value: 1,
+          Status: "default",
+        },
+        "pdfjs.sidebarViewOnLoad": {
+          Value: 0,
+          Status: "default",
+        },
+      },
+    },
+    unlockedPrefs: {
+      "pdfjs.cursorToolOnLoad": 1,
+      "pdfjs.sidebarViewOnLoad": 0,
+    },
+  },
+
+  // Bug 1772503
+  {
+    policies: {
+      DisableFirefoxStudies: true,
+    },
+    lockedPrefs: {
+      "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.addons": false,
+      "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.features": false,
+    },
+  },
+  {
+    policies: {
+      Preferences: {
+        "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.addons": {
+          Value: true,
+        },
+        "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.features": {
+          Value: true,
+        },
+      },
+    },
+    lockedPrefs: {
+      "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.addons": true,
+      "browser.newtabpage.activity-stream.asrouter.userprefs.cfr.features": true,
+    },
+  },
+
+  // POLICY: AutofillAddressEnabled, AutofillCreditCardEnabled
+
+  {
+    policies: {
+      AutofillAddressEnabled: false,
+      AutofillCreditCardEnabled: false,
+    },
+    lockedPrefs: {
+      "extensions.formautofill.addresses.enabled": false,
+      "extensions.formautofill.creditCards.enabled": false,
+    },
+  },
+
+  // POLICY: Proxy - locking if no values are set
+  {
+    policies: {
+      Proxy: {
+        Locked: true,
+      },
+    },
+    lockedPrefs: {
+      "network.proxy.type": 5,
+    },
+  },
+
+  // POLICY: DisableEncryptedClientHello
+  {
+    policies: {
+      DisableEncryptedClientHello: true,
+    },
+    lockedPrefs: {
+      "network.dns.echconfig.enabled": false,
+      "network.dns.http3_echconfig.enabled": false,
+    },
+  },
+
+  // POLICY: PostQuantumKeyAgreementEnabled
+  {
+    policies: {
+      PostQuantumKeyAgreementEnabled: false,
+    },
+    lockedPrefs: {
+      "network.http.http3.enable_kyber": false,
+      "security.tls.enable_kyber": false,
+    },
+  },
+
+  // POLICY: HttpsOnlyMode
+  {
+    policies: {
+      HttpsOnlyMode: "enabled",
+    },
+    unlockedPrefs: {
+      "dom.security.https_only_mode": true,
+    },
+  },
+
+  {
+    policies: {
+      HttpsOnlyMode: "disallowed",
+    },
+    lockedPrefs: {
+      "dom.security.https_only_mode": false,
+    },
+  },
+
+  {
+    policies: {
+      HttpsOnlyMode: "force_enabled",
+    },
+    lockedPrefs: {
+      "dom.security.https_only_mode": true,
+    },
+  },
 ];
 
 add_task(async function test_policy_simple_prefs() {
