@@ -8,9 +8,13 @@
 
 #include "gfx2DGlue.h"
 #include "gfxUtils.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/Element.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_mathml.h"
 #include "mozilla/gfx/2D.h"
 #include "mozilla/gfx/PathHelpers.h"
+#include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 #include "nsWhitespaceTokenizer.h"
 
@@ -58,7 +62,7 @@ nsMathMLmencloseFrame::nsMathMLmencloseFrame(ComputedStyle* aStyle,
       mRadicalCharIndex(-1),
       mContentWidth(0) {}
 
-nsMathMLmencloseFrame::~nsMathMLmencloseFrame() {}
+nsMathMLmencloseFrame::~nsMathMLmencloseFrame() = default;
 
 nsresult nsMathMLmencloseFrame::AllocateMathMLChar(nsMencloseNotation mask) {
   // Is the char already allocated?
@@ -72,7 +76,9 @@ nsresult nsMathMLmencloseFrame::AllocateMathMLChar(nsMencloseNotation mask) {
   uint32_t i = mMathMLChar.Length();
   nsAutoString Char;
 
-  if (!mMathMLChar.AppendElement()) return NS_ERROR_OUT_OF_MEMORY;
+  // XXX(Bug 1631371) Check if this should use a fallible operation as it
+  // pretended earlier, or change the return type to void.
+  mMathMLChar.AppendElement();
 
   if (mask == NOTATION_LONGDIV) {
     Char.Assign(kLongDivChar);
@@ -82,10 +88,8 @@ nsresult nsMathMLmencloseFrame::AllocateMathMLChar(nsMencloseNotation mask) {
     mRadicalCharIndex = i;
   }
 
-  nsPresContext* presContext = PresContext();
   mMathMLChar[i].SetData(Char);
-  ResolveMathMLCharStyle(presContext, mContent, mComputedStyle,
-                         &mMathMLChar[i]);
+  mMathMLChar[i].SetComputedStyle(Style());
 
   return NS_OK;
 }
@@ -104,10 +108,6 @@ nsresult nsMathMLmencloseFrame::AddNotation(const nsAString& aNotation) {
   } else if (aNotation.EqualsLiteral("actuarial")) {
     mNotationsToDraw += NOTATION_RIGHT;
     mNotationsToDraw += NOTATION_TOP;
-  } else if (aNotation.EqualsLiteral("radical")) {
-    rv = AllocateMathMLChar(NOTATION_RADICAL);
-    NS_ENSURE_SUCCESS(rv, rv);
-    mNotationsToDraw += NOTATION_RADICAL;
   } else if (aNotation.EqualsLiteral("box")) {
     mNotationsToDraw += NOTATION_LEFT;
     mNotationsToDraw += NOTATION_RIGHT;
@@ -157,8 +157,7 @@ void nsMathMLmencloseFrame::InitNotations() {
 
   nsAutoString value;
 
-  if (mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::notation_,
-                                     value)) {
+  if (mContent->AsElement()->GetAttr(nsGkAtoms::notation_, value)) {
     // parse the notation attribute
     nsWhitespaceTokenizer tokenizer(value);
 
@@ -207,8 +206,6 @@ void nsMathMLmencloseFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
   // paint the menclosed content
   nsMathMLContainerFrame::BuildDisplayList(aBuilder, aLists);
 
-  if (NS_MATHML_HAS_ERROR(mPresentationData.flags)) return;
-
   nsRect mencloseRect = nsIFrame::GetRect();
   mencloseRect.x = mencloseRect.y = 0;
 
@@ -217,7 +214,10 @@ void nsMathMLmencloseFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
 
     nsRect rect;
     mMathMLChar[mRadicalCharIndex].GetRect(rect);
-    rect.MoveBy(StyleVisibility()->mDirection ? -mContentWidth : rect.width, 0);
+    rect.MoveBy(StyleVisibility()->mDirection == StyleDirection::Rtl
+                    ? -mContentWidth
+                    : rect.width,
+                0);
     rect.SizeTo(mContentWidth, mRadicalRuleThickness);
     DisplayBar(aBuilder, this, rect, aLists, NOTATION_RADICAL);
   }
@@ -320,7 +320,7 @@ nsresult nsMathMLmencloseFrame::PlaceInternal(DrawTarget* aDrawTarget,
   ReflowOutput baseSize(aDesiredSize.GetWritingMode());
   nsresult rv = nsMathMLContainerFrame::Place(aDrawTarget, false, baseSize);
 
-  if (NS_MATHML_HAS_ERROR(mPresentationData.flags) || NS_FAILED(rv)) {
+  if (NS_FAILED(rv)) {
     DidReflowChildren(PrincipalChildList().FirstChild());
     return rv;
   }
@@ -359,9 +359,8 @@ nsresult nsMathMLmencloseFrame::PlaceInternal(DrawTarget* aDrawTarget,
   if (delta) padding += onePixel - delta;  // round up
 
   if (IsToDraw(NOTATION_LONGDIV) || IsToDraw(NOTATION_RADICAL)) {
-    GetRadicalParameters(
-        fm, StyleFont()->mMathDisplay == NS_MATHML_DISPLAYSTYLE_BLOCK,
-        mRadicalRuleThickness, leading, psi);
+    GetRadicalParameters(fm, StyleFont()->mMathStyle == StyleMathStyle::Normal,
+                         mRadicalRuleThickness, leading, psi);
 
     // make sure that the rule appears on on screen
     if (mRadicalRuleThickness < onePixel) {
@@ -500,7 +499,9 @@ nsresult nsMathMLmencloseFrame::PlaceInternal(DrawTarget* aDrawTarget,
   ///////////////
   // radical notation:
   if (IsToDraw(NOTATION_RADICAL)) {
-    nscoord* dx_leading = StyleVisibility()->mDirection ? &dx_right : &dx_left;
+    nscoord* dx_leading = StyleVisibility()->mDirection == StyleDirection::Rtl
+                              ? &dx_right
+                              : &dx_left;
 
     if (aWidthOnly) {
       nscoord radical_width = mMathMLChar[mRadicalCharIndex].GetMaxWidth(
@@ -519,7 +520,7 @@ nsresult nsMathMLmencloseFrame::PlaceInternal(DrawTarget* aDrawTarget,
       mMathMLChar[mRadicalCharIndex].Stretch(
           this, aDrawTarget, fontSizeInflation, NS_STRETCH_DIRECTION_VERTICAL,
           contSize, bmRadicalChar, NS_STRETCH_LARGER,
-          StyleVisibility()->mDirection);
+          StyleVisibility()->mDirection == StyleDirection::Rtl);
       mMathMLChar[mRadicalCharIndex].GetBoundingMetrics(bmRadicalChar);
 
       // Update horizontal parameters
@@ -636,9 +637,9 @@ nsresult nsMathMLmencloseFrame::PlaceInternal(DrawTarget* aDrawTarget,
           bmLongdivChar.ascent + bmLongdivChar.descent));
 
     if (IsToDraw(NOTATION_RADICAL)) {
-      nscoord dx =
-          (StyleVisibility()->mDirection ? dx_left + bmBase.width
-                                         : dx_left - bmRadicalChar.width);
+      nscoord dx = (StyleVisibility()->mDirection == StyleDirection::Rtl
+                        ? dx_left + bmBase.width
+                        : dx_left - bmRadicalChar.width);
 
       mMathMLChar[mRadicalCharIndex].SetRect(nsRect(
           dx, aDesiredSize.BlockStartAscent() - radicalAscent,
@@ -682,24 +683,16 @@ nsresult nsMathMLmencloseFrame::AttributeChanged(int32_t aNameSpaceID,
                                                   aModType);
 }
 
-//////////////////
-// the Style System will use these to pass the proper ComputedStyle to our
-// MathMLChar
-ComputedStyle* nsMathMLmencloseFrame::GetAdditionalComputedStyle(
-    int32_t aIndex) const {
-  int32_t len = mMathMLChar.Length();
-  if (aIndex >= 0 && aIndex < len)
-    return mMathMLChar[aIndex].GetComputedStyle();
-  else
-    return nullptr;
+void nsMathMLmencloseFrame::DidSetComputedStyle(ComputedStyle* aOldStyle) {
+  nsMathMLContainerFrame::DidSetComputedStyle(aOldStyle);
+  for (auto& ch : mMathMLChar) {
+    ch.SetComputedStyle(Style());
+  }
 }
 
-void nsMathMLmencloseFrame::SetAdditionalComputedStyle(
-    int32_t aIndex, ComputedStyle* aComputedStyle) {
-  int32_t len = mMathMLChar.Length();
-  if (aIndex >= 0 && aIndex < len)
-    mMathMLChar[aIndex].SetComputedStyle(aComputedStyle);
-}
+//////////////////
+
+namespace mozilla {
 
 class nsDisplayNotation final : public nsPaintedDisplayItem {
  public:
@@ -712,11 +705,7 @@ class nsDisplayNotation final : public nsPaintedDisplayItem {
         mType(aType) {
     MOZ_COUNT_CTOR(nsDisplayNotation);
   }
-#ifdef NS_BUILD_REFCNT_LOGGING
-  virtual ~nsDisplayNotation() { MOZ_COUNT_DTOR(nsDisplayNotation); }
-#endif
-
-  virtual uint16_t CalculatePerFrameKey() const override { return mType; }
+  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplayNotation)
 
   virtual void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override;
   NS_DISPLAY_DECL_NAME("MathMLMencloseNotation", TYPE_MATHML_MENCLOSE_NOTATION)
@@ -818,6 +807,8 @@ void nsDisplayNotation::Paint(nsDisplayListBuilder* aBuilder,
   }
 }
 
+}  // namespace mozilla
+
 void nsMathMLmencloseFrame::DisplayNotation(nsDisplayListBuilder* aBuilder,
                                             nsIFrame* aFrame,
                                             const nsRect& aRect,
@@ -828,6 +819,7 @@ void nsMathMLmencloseFrame::DisplayNotation(nsDisplayListBuilder* aBuilder,
       aThickness <= 0)
     return;
 
-  aLists.Content()->AppendNewToTop<nsDisplayNotation>(aBuilder, aFrame, aRect,
-                                                      aThickness, aType);
+  const uint16_t index = aType;
+  aLists.Content()->AppendNewToTopWithIndex<nsDisplayNotation>(
+      aBuilder, aFrame, index, aRect, aThickness, aType);
 }
