@@ -6,7 +6,7 @@
 #include "nsCellMap.h"
 
 #include "mozilla/PresShell.h"
-
+#include "mozilla/StaticPtr.h"
 #include "nsTArray.h"
 #include "nsTableFrame.h"
 #include "nsTableCellFrame.h"
@@ -30,7 +30,7 @@ static void SetDamageArea(int32_t aStartCol, int32_t aStartRow,
 }
 
 // Empty static array used for SafeElementAt() calls on mRows.
-static nsCellMap::CellDataArray* sEmptyRow;
+static StaticAutoPtr<nsCellMap::CellDataArray> sEmptyRow;
 
 // CellData
 
@@ -55,8 +55,7 @@ nsTableCellMap::nsTableCellMap(nsTableFrame& aTableFrame, bool aBorderCollapse)
     : mTableFrame(aTableFrame), mFirstMap(nullptr), mBCInfo(nullptr) {
   MOZ_COUNT_CTOR(nsTableCellMap);
 
-  nsTableFrame::RowGroupArray orderedRowGroups;
-  aTableFrame.OrderRowGroups(orderedRowGroups);
+  nsTableFrame::RowGroupArray orderedRowGroups = aTableFrame.OrderedRowGroups();
 
   nsTableRowGroupFrame* prior = nullptr;
   for (uint32_t rgX = 0; rgX < orderedRowGroups.Length(); rgX++) {
@@ -206,31 +205,48 @@ nsCellMap* nsTableCellMap::GetMapFor(const nsTableRowGroupFrame* aRowGroup,
     return map;
   }
 
-  // if aRowGroup is a repeated header or footer find the header or footer it
-  // was repeated from
-  if (aRowGroup->IsRepeatable()) {
-    nsTableFrame* fifTable =
-        static_cast<nsTableFrame*>(mTableFrame.FirstInFlow());
-
-    const nsStyleDisplay* display = aRowGroup->StyleDisplay();
-    nsTableRowGroupFrame* rgOrig =
-        (StyleDisplay::TableHeaderGroup == display->mDisplay)
-            ? fifTable->GetTHead()
-            : fifTable->GetTFoot();
-    // find the row group cell map using the original header/footer
-    if (rgOrig && rgOrig != aRowGroup) {
+  // If aRowGroup is a repeated header or footer find the header or footer it
+  // was repeated from.
+  // Bug 1442018: we also need this search for header/footer frames that are
+  // not marked as _repeatable_ because they have a next-in-flow, as they may
+  // nevertheless have been _repeated_ from an earlier fragment.
+  auto isTableHeaderFooterGroup = [](const nsTableRowGroupFrame* aRG) -> bool {
+    const auto display = aRG->StyleDisplay()->mDisplay;
+    return display == StyleDisplay::TableHeaderGroup ||
+           display == StyleDisplay::TableFooterGroup;
+  };
+  if (aRowGroup->IsRepeatable() ||
+      (aRowGroup->GetNextInFlow() && isTableHeaderFooterGroup(aRowGroup))) {
+    auto findOtherRowGroupOfType =
+        [aRowGroup](nsTableFrame* aTable) -> nsTableRowGroupFrame* {
+      const auto display = aRowGroup->StyleDisplay()->mDisplay;
+      auto* table = aTable->FirstContinuation();
+      for (; table; table = table->GetNextContinuation()) {
+        for (auto* child : table->PrincipalChildList()) {
+          if (child->StyleDisplay()->mDisplay == display &&
+              child != aRowGroup) {
+            return static_cast<nsTableRowGroupFrame*>(child);
+          }
+        }
+      }
+      return nullptr;
+    };
+    if (auto* rgOrig = findOtherRowGroupOfType(&mTableFrame)) {
       return GetMapFor(rgOrig, aStartHint);
     }
+    MOZ_ASSERT_UNREACHABLE(
+        "A repeated header/footer should always have an "
+        "original header/footer it was repeated from");
   }
 
   return nullptr;
 }
 
 void nsTableCellMap::Synchronize(nsTableFrame* aTableFrame) {
-  nsTableFrame::RowGroupArray orderedRowGroups;
   AutoTArray<nsCellMap*, 8> maps;
 
-  aTableFrame->OrderRowGroups(orderedRowGroups);
+  nsTableFrame::RowGroupArray orderedRowGroups =
+      aTableFrame->OrderedRowGroups();
   if (!orderedRowGroups.Length()) {
     return;
   }
@@ -246,12 +262,9 @@ void nsTableCellMap::Synchronize(nsTableFrame* aTableFrame) {
     map = GetMapFor(static_cast<nsTableRowGroupFrame*>(rgFrame->FirstInFlow()),
                     map);
     if (map) {
-      if (!maps.AppendElement(map)) {
-        delete map;
-        map = nullptr;
-        NS_WARNING("Could not AppendElement");
-        break;
-      }
+      // XXX(Bug 1631371) Check if this should use a fallible operation as it
+      // pretended earlier, or change the return type to void.
+      maps.AppendElement(map);
     }
   }
   if (maps.IsEmpty()) {
@@ -374,13 +387,13 @@ CellData* nsTableCellMap::GetDataAt(int32_t aRowIndex,
 }
 
 void nsTableCellMap::AddColsAtEnd(uint32_t aNumCols) {
-  if (!mCols.AppendElements(aNumCols)) {
-    NS_WARNING("Could not AppendElement");
-  }
+  // XXX(Bug 1631371) Check if this should use a fallible operation as it
+  // pretended earlier.
+  mCols.AppendElements(aNumCols);
   if (mBCInfo) {
-    if (!mBCInfo->mBEndBorders.AppendElements(aNumCols)) {
-      NS_WARNING("Could not AppendElement");
-    }
+    // XXX(Bug 1631371) Check if this should use a fallible operation as it
+    // pretended earlier.
+    mBCInfo->mBEndBorders.AppendElements(aNumCols);
   }
 }
 
@@ -390,8 +403,8 @@ void nsTableCellMap::RemoveColsAtEnd() {
   // eColAnonymousCell
   int32_t numCols = GetColCount();
   int32_t lastGoodColIndex = mTableFrame.GetIndexOfLastRealCol();
-  for (int32_t colX = numCols - 1; (colX >= 0) && (colX > lastGoodColIndex);
-       colX--) {
+  MOZ_ASSERT(lastGoodColIndex >= -1);
+  for (int32_t colX = numCols - 1; colX > lastGoodColIndex; colX--) {
     nsColInfo& colInfo = mCols.ElementAt(colX);
     if ((colInfo.mNumCellsOrig <= 0) && (colInfo.mNumCellsSpan <= 0)) {
       mCols.RemoveElementAt(colX);
@@ -665,7 +678,8 @@ void nsTableCellMap::Dump(char* aString) const {
           printf("l=%d%X%d ", int32_t(size), owner, segStart);
         } else {
           size = cd.GetCorner(side, bevel);
-          printf("c=%d%X%d ", int32_t(size), side, bevel);
+          printf("c=%d%hhX%d ", int32_t(size), static_cast<uint8_t>(side),
+                 bevel);
         }
       }
       BCData& cd = mBCInfo->mBEndIEndCorner;
@@ -677,7 +691,7 @@ void nsTableCellMap::Dump(char* aString) const {
         printf("l=%d%X%d ", int32_t(size), owner, segStart);
       } else {
         size = cd.GetCorner(side, bevel);
-        printf("c=%d%X%d ", int32_t(size), side, bevel);
+        printf("c=%d%hhX%d ", int32_t(size), static_cast<uint8_t>(side), bevel);
       }
     }
     printf("\n");
@@ -805,7 +819,7 @@ bool nsTableCellMap::RowHasSpanningCells(int32_t aRowIndex,
   return false;
 }
 
-// FIXME: The only value callers pass for aSide is eLogicalSideBEnd.
+// FIXME: The only value callers pass for aSide is LogicalSide::BEnd.
 // Consider removing support for the other three values.
 void nsTableCellMap::ResetBStartStart(LogicalSide aSide, nsCellMap& aCellMap,
                                       uint32_t aRowGroupStart,
@@ -816,16 +830,16 @@ void nsTableCellMap::ResetBStartStart(LogicalSide aSide, nsCellMap& aCellMap,
   BCData* bcData = nullptr;
 
   switch (aSide) {
-    case eLogicalSideBEnd:
+    case LogicalSide::BEnd:
       aRowIndex++;
-      MOZ_FALLTHROUGH;
-    case eLogicalSideBStart:
+      [[fallthrough]];
+    case LogicalSide::BStart:
       cellData = (BCCellData*)aCellMap.GetDataAt(aRowIndex - aRowGroupStart,
                                                  aColIndex);
       if (cellData) {
         bcData = &cellData->mData;
       } else {
-        NS_ASSERTION(aSide == eLogicalSideBEnd, "program error");
+        NS_ASSERTION(aSide == LogicalSide::BEnd, "program error");
         // try the next row group
         nsCellMap* cellMap = aCellMap.GetNextSibling();
         if (cellMap) {
@@ -838,16 +852,16 @@ void nsTableCellMap::ResetBStartStart(LogicalSide aSide, nsCellMap& aCellMap,
         }
       }
       break;
-    case eLogicalSideIEnd:
+    case LogicalSide::IEnd:
       aColIndex++;
-      MOZ_FALLTHROUGH;
-    case eLogicalSideIStart:
+      [[fallthrough]];
+    case LogicalSide::IStart:
       cellData = (BCCellData*)aCellMap.GetDataAt(aRowIndex - aRowGroupStart,
                                                  aColIndex);
       if (cellData) {
         bcData = &cellData->mData;
       } else {
-        NS_ASSERTION(aSide == eLogicalSideIEnd, "program error");
+        NS_ASSERTION(aSide == LogicalSide::IEnd, "program error");
         bcData = GetIEndMostBorder(aRowIndex);
       }
       break;
@@ -877,11 +891,11 @@ void nsTableCellMap::SetBCBorderEdge(LogicalSide aSide, nsCellMap& aCellMap,
   bool changed;
 
   switch (aSide) {
-    case eLogicalSideBEnd:
+    case LogicalSide::BEnd:
       rgYPos++;
       yPos++;
-      MOZ_FALLTHROUGH;
-    case eLogicalSideBStart:
+      [[fallthrough]];
+    case LogicalSide::BStart:
       lastIndex = xPos + aLength - 1;
       for (xIndex = xPos; xIndex <= lastIndex; xIndex++) {
         changed = aChanged && (xIndex == xPos);
@@ -895,7 +909,7 @@ void nsTableCellMap::SetBCBorderEdge(LogicalSide aSide, nsCellMap& aCellMap,
                                                         false, 0, damageArea);
             if (!cellData) ABORT0();
           } else {
-            NS_ASSERTION(aSide == eLogicalSideBEnd, "program error");
+            NS_ASSERTION(aSide == LogicalSide::BEnd, "program error");
             // try the next non empty row group
             nsCellMap* cellMap = aCellMap.GetNextSibling();
             while (cellMap && (0 == cellMap->GetRowCount())) {
@@ -922,10 +936,10 @@ void nsTableCellMap::SetBCBorderEdge(LogicalSide aSide, nsCellMap& aCellMap,
           NS_ERROR("Cellmap: BStart edge not found");
       }
       break;
-    case eLogicalSideIEnd:
+    case LogicalSide::IEnd:
       xPos++;
-      MOZ_FALLTHROUGH;
-    case eLogicalSideIStart:
+      [[fallthrough]];
+    case LogicalSide::IStart:
       // since bStart, bEnd borders were set, there should already be a cellData
       // entry
       lastIndex = rgYPos + aLength - 1;
@@ -935,7 +949,7 @@ void nsTableCellMap::SetBCBorderEdge(LogicalSide aSide, nsCellMap& aCellMap,
         if (cellData) {
           cellData->mData.SetIStartEdge(aOwner, aSize, changed);
         } else {
-          NS_ASSERTION(aSide == eLogicalSideIEnd, "program error");
+          NS_ASSERTION(aSide == LogicalSide::IEnd, "program error");
           BCData* bcData = GetIEndMostBorder(yIndex + aCellMapStart);
           if (bcData) {
             bcData->SetIStartEdge(aOwner, aSize, changed);
@@ -968,13 +982,13 @@ void nsTableCellMap::SetBCBorderCorner(LogicalCorner aCorner,
   int32_t yPos = aRowIndex;
   int32_t rgYPos = aRowIndex - aCellMapStart;
 
-  if (eLogicalCornerBStartIEnd == aCorner) {
+  if (LogicalCorner::BStartIEnd == aCorner) {
     xPos++;
-  } else if (eLogicalCornerBEndIEnd == aCorner) {
+  } else if (LogicalCorner::BEndIEnd == aCorner) {
     xPos++;
     rgYPos++;
     yPos++;
-  } else if (eLogicalCornerBEndIStart == aCorner) {
+  } else if (LogicalCorner::BEndIStart == aCorner) {
     rgYPos++;
     yPos++;
   }
@@ -1053,10 +1067,7 @@ void nsCellMap::Init() {
 }
 
 /* static */
-void nsCellMap::Shutdown() {
-  delete sEmptyRow;
-  sEmptyRow = nullptr;
-}
+void nsCellMap::Shutdown() { sEmptyRow = nullptr; }
 
 nsTableCellFrame* nsCellMap::GetCellFrame(int32_t aRowIndexIn,
                                           int32_t aColIndexIn, CellData& aData,
@@ -1169,7 +1180,10 @@ bool nsCellMap::Grow(nsTableCellMap& aMap, int32_t aNumRows,
   uint32_t startRowIndex = (aRowIndex >= 0) ? aRowIndex : mRows.Length();
   NS_ASSERTION(startRowIndex <= mRows.Length(), "Missing grow call inbetween");
 
-  return mRows.InsertElementsAt(startRowIndex, aNumRows, numCols) != nullptr;
+  // XXX Change the return type of this function to void, or use a fallible
+  // operation.
+  mRows.InsertElementsAt(startRowIndex, aNumRows, numCols);
+  return true;
 }
 
 void nsCellMap::GrowRow(CellDataArray& aRow, int32_t aNumCols)
@@ -1415,15 +1429,13 @@ CellData* nsCellMap::AppendCell(nsTableCellMap& aMap,
 bool nsCellMap::CellsSpanOut(nsTArray<nsTableRowFrame*>& aRows) const {
   int32_t numNewRows = aRows.Length();
   for (int32_t rowX = 0; rowX < numNewRows; rowX++) {
-    nsIFrame* rowFrame = (nsIFrame*)aRows.ElementAt(rowX);
-    for (nsIFrame* childFrame : rowFrame->PrincipalChildList()) {
-      nsTableCellFrame* cellFrame = do_QueryFrame(childFrame);
-      if (cellFrame) {
-        bool zeroSpan;
-        int32_t rowSpan = GetRowSpanForNewCell(cellFrame, rowX, zeroSpan);
-        if (zeroSpan || rowX + rowSpan > numNewRows) {
-          return true;
-        }
+    nsTableRowFrame* rowFrame = aRows.ElementAt(rowX);
+    for (nsTableCellFrame* cellFrame = rowFrame->GetFirstCell(); cellFrame;
+         cellFrame = cellFrame->GetNextCell()) {
+      bool zeroSpan;
+      int32_t rowSpan = GetRowSpanForNewCell(cellFrame, rowX, zeroSpan);
+      if (zeroSpan || rowX + rowSpan > numNewRows) {
+        return true;
       }
     }
   }
@@ -1581,12 +1593,10 @@ void nsCellMap::ExpandWithRows(nsTableCellMap& aMap,
     nsTableRowFrame* rFrame = aRowFrames.ElementAt(newRowIndex);
     // append cells
     int32_t colIndex = 0;
-    for (nsIFrame* cFrame : rFrame->PrincipalChildList()) {
-      nsTableCellFrame* cellFrame = do_QueryFrame(cFrame);
-      if (cellFrame) {
-        AppendCell(aMap, cellFrame, rowX, false, aRgFirstRowIndex, aDamageArea,
-                   &colIndex);
-      }
+    for (nsTableCellFrame* cellFrame = rFrame->GetFirstCell(); cellFrame;
+         cellFrame = cellFrame->GetNextCell()) {
+      AppendCell(aMap, cellFrame, rowX, false, aRgFirstRowIndex, aDamageArea,
+                 &colIndex);
     }
     newRowIndex++;
   }
@@ -1637,16 +1647,8 @@ void nsCellMap::ExpandWithCells(nsTableCellMap& aMap,
       if (insertionIndex > startColIndex) {
         insertionIndex = startColIndex;
       }
-      if (!row.InsertElementsAt(insertionIndex,
-                                endColIndex - insertionIndex + 1,
-                                (CellData*)nullptr) &&
-          rowX == aRowIndex) {
-        // Failed to insert the slots, and this is the very first row.  That
-        // means that we need to clean up |origData| before returning, since
-        // the cellmap doesn't own it yet.
-        DestroyCellData(origData);
-        return;
-      }
+      row.InsertElementsAt(insertionIndex, endColIndex - insertionIndex + 1,
+                           (CellData*)nullptr);
 
       for (int32_t colX = startColIndex; colX <= endColIndex; colX++) {
         CellData* data = origData;
@@ -1816,13 +1818,16 @@ int32_t nsCellMap::GetRowSpanForNewCell(nsTableCellFrame* aCellFrameToAdd,
 bool nsCellMap::HasMoreThanOneCell(int32_t aRowIndex) const {
   const CellDataArray& row = mRows.SafeElementAt(aRowIndex, *sEmptyRow);
   uint32_t maxColIndex = row.Length();
-  uint32_t count = 0;
   uint32_t colIndex;
+  bool foundOne = false;
   for (colIndex = 0; colIndex < maxColIndex; colIndex++) {
     CellData* cellData = row[colIndex];
-    if (cellData && (cellData->GetCellFrame() || cellData->IsRowSpan()))
-      count++;
-    if (count > 1) return true;
+    if (cellData && (cellData->GetCellFrame() || cellData->IsRowSpan())) {
+      if (foundOne) {
+        return true;
+      }
+      foundOne = true;
+    }
   }
   return false;
 }
@@ -1868,7 +1873,6 @@ void nsCellMap::ShrinkWithoutCell(nsTableCellMap& aMap,
 
   // get the rowspan and colspan from the cell map since the content may have
   // changed
-  uint32_t numCols = aMap.GetColCount();
   int32_t rowSpan = GetRowSpan(aRowIndex, aColIndex, true);
   uint32_t colSpan = GetEffectiveColSpan(aMap, aRowIndex, aColIndex);
   uint32_t endRowIndex = aRowIndex + rowSpan - 1;
@@ -1902,7 +1906,7 @@ void nsCellMap::ShrinkWithoutCell(nsTableCellMap& aMap,
     }
   }
 
-  numCols = aMap.GetColCount();
+  uint32_t numCols = aMap.GetColCount();
 
   // update the row and col info due to shifting
   for (rowX = aRowIndex; rowX <= endRowIndex; rowX++) {
@@ -1951,8 +1955,7 @@ void nsCellMap::RebuildConsideringRows(
   NS_ASSERTION(!!aMap.mBCInfo == mIsBC, "BC state mismatch");
   // copy the old cell map into a new array
   uint32_t numOrigRows = mRows.Length();
-  nsTArray<CellDataArray> origRows;
-  mRows.SwapElements(origRows);
+  nsTArray<CellDataArray> origRows = std::move(mRows);
 
   int32_t rowNumberChange;
   if (aRowsToInsert) {
@@ -2004,11 +2007,9 @@ void nsCellMap::RebuildConsideringRows(
     int32_t numNewRows = aRowsToInsert->Length();
     for (int32_t newRowX = 0; newRowX < numNewRows; newRowX++) {
       nsTableRowFrame* rFrame = aRowsToInsert->ElementAt(newRowX);
-      for (nsIFrame* cFrame : rFrame->PrincipalChildList()) {
-        nsTableCellFrame* cellFrame = do_QueryFrame(cFrame);
-        if (cellFrame) {
-          AppendCell(aMap, cellFrame, rowX, false, 0, damageArea);
-        }
+      for (nsTableCellFrame* cellFrame = rFrame->GetFirstCell(); cellFrame;
+           cellFrame = cellFrame->GetNextCell()) {
+        AppendCell(aMap, cellFrame, rowX, false, 0, damageArea);
       }
       rowX++;
     }
@@ -2051,8 +2052,7 @@ void nsCellMap::RebuildConsideringCells(
   NS_ASSERTION(!!aMap.mBCInfo == mIsBC, "BC state mismatch");
   // copy the old cell map into a new array
   int32_t numOrigRows = mRows.Length();
-  nsTArray<CellDataArray> origRows;
-  mRows.SwapElements(origRows);
+  nsTArray<CellDataArray> origRows = std::move(mRows);
 
   int32_t numNewCells = (aCellFrames) ? aCellFrames->Length() : 0;
 
@@ -2145,8 +2145,11 @@ void nsCellMap::RemoveCell(nsTableCellMap& aMap, nsTableCellFrame* aCellFrame,
   // originating cells, we need to assume that this the only such cell, and
   // rebuild so that there are no extraneous cols at the end. The same is true
   // for removing rows.
-  if (!aCellFrame->GetRowSpan() || !aCellFrame->GetColSpan())
-    spansCauseRebuild = true;
+  if (!spansCauseRebuild) {
+    if (!aCellFrame->GetRowSpan() || !aCellFrame->GetColSpan()) {
+      spansCauseRebuild = true;
+    }
+  }
 
   if (spansCauseRebuild) {
     aMap.RebuildConsideringCells(this, nullptr, aRowIndex, startColIndex, false,
@@ -2162,14 +2165,14 @@ void nsCellMap::Dump(bool aIsBorderCollapse) const {
   printf("\n  ***** START GROUP CELL MAP DUMP ***** %p\n", (void*)this);
   nsTableRowGroupFrame* rg = GetRowGroup();
   const nsStyleDisplay* display = rg->StyleDisplay();
-  switch (display->mDisplay) {
-    case StyleDisplay::TableHeaderGroup:
+  switch (display->DisplayInside()) {
+    case StyleDisplayInside::TableHeaderGroup:
       printf("  thead ");
       break;
-    case StyleDisplay::TableFooterGroup:
+    case StyleDisplayInside::TableFooterGroup:
       printf("  tfoot ");
       break;
-    case StyleDisplay::TableRowGroup:
+    case StyleDisplayInside::TableRowGroup:
       printf("  tbody ");
       break;
     default:
@@ -2223,7 +2226,8 @@ void nsCellMap::Dump(bool aIsBorderCollapse) const {
               printf("l=%d%d%d ", int32_t(size), owner, segStart);
             } else {
               size = cd->mData.GetCorner(side, bevel);
-              printf("c=%d%d%d ", int32_t(size), side, bevel);
+              printf("c=%d%hhu%d ", int32_t(size), static_cast<uint8_t>(side),
+                     bevel);
             }
           }
         }
@@ -2233,7 +2237,6 @@ void nsCellMap::Dump(bool aIsBorderCollapse) const {
   }
 
   // output info mapping Ci,j to cell address
-  uint32_t cellCount = 0;
   for (uint32_t rIndex = 0; rIndex < mapRowCount; rIndex++) {
     const CellDataArray& row = mRows[rIndex];
     uint32_t colCount = row.Length();
@@ -2246,7 +2249,6 @@ void nsCellMap::Dump(bool aIsBorderCollapse) const {
           uint32_t cellFrameColIndex = cellFrame->ColIndex();
           printf("C%d,%d=%p(%u)  ", rIndex, colIndex, (void*)cellFrame,
                  cellFrameColIndex);
-          cellCount++;
         }
       }
     }
@@ -2453,7 +2455,7 @@ nsTableCellFrame* nsCellMapColumnIterator::GetNextFrame(int32_t* aRow,
     return nullptr;
   }
 
-  while (1) {
+  while (true) {
     NS_ASSERTION(mCurMapRow < mCurMapRelevantRowCount, "Bogus mOrigCells?");
     // Safe to just get the row (which is faster than calling GetDataAt(), but
     // there may not be that many cells in it, so have to use SafeElementAt for
