@@ -13,6 +13,7 @@
 #include "nsIContent.h"
 #include "mozilla/ComputedStyle.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_layout.h"
 
 using namespace mozilla;
 
@@ -36,9 +37,6 @@ nsTableColFrame::nsTableColFrame(ComputedStyle* aStyle,
       mColIndex(0),
       mIStartBorderWidth(0),
       mIEndBorderWidth(0),
-      mBStartContBorderWidth(0),
-      mIEndContBorderWidth(0),
-      mBEndContBorderWidth(0),
       mHasSpecifiedCoord(false) {
   SetColType(eColContent);
   ResetIntrinsics();
@@ -49,7 +47,7 @@ nsTableColFrame::nsTableColFrame(ComputedStyle* aStyle,
 nsTableColFrame::~nsTableColFrame() = default;
 
 nsTableColType nsTableColFrame::GetColType() const {
-  return (nsTableColType)((mState & COL_TYPE_BITS) >> COL_TYPE_OFFSET);
+  return (nsTableColType)((GetStateBits() & COL_TYPE_BITS) >> COL_TYPE_OFFSET);
 }
 
 void nsTableColFrame::SetColType(nsTableColType aType) {
@@ -78,30 +76,12 @@ void nsTableColFrame::DidSetComputedStyle(ComputedStyle* aOldComputedStyle) {
   }
 }
 
-void nsTableColFrame::SetContinuousBCBorderWidth(LogicalSide aForSide,
-                                                 BCPixelSize aPixelValue) {
-  switch (aForSide) {
-    case eLogicalSideBStart:
-      mBStartContBorderWidth = aPixelValue;
-      return;
-    case eLogicalSideIEnd:
-      mIEndContBorderWidth = aPixelValue;
-      return;
-    case eLogicalSideBEnd:
-      mBEndContBorderWidth = aPixelValue;
-      return;
-    default:
-      NS_ERROR("invalid side arg");
-  }
-}
-
 void nsTableColFrame::Reflow(nsPresContext* aPresContext,
                              ReflowOutput& aDesiredSize,
                              const ReflowInput& aReflowInput,
                              nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsTableColFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
   aDesiredSize.ClearSize();
   const nsStyleVisibility* colVis = StyleVisibility();
@@ -109,7 +89,6 @@ void nsTableColFrame::Reflow(nsPresContext* aPresContext,
   if (collapseCol) {
     GetTableFrame()->SetNeedToCollapse(true);
   }
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aDesiredSize);
 }
 
 void nsTableColFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
