@@ -6,6 +6,8 @@ use crate::attr::CaseSensitivity;
 use crate::bloom::BloomFilter;
 use crate::nth_index_cache::{NthIndexCache, NthIndexCacheInner};
 use crate::parser::{Selector, SelectorImpl};
+use crate::relative_selector::cache::RelativeSelectorCache;
+use crate::relative_selector::filter::RelativeSelectorFilterMap;
 use crate::tree::{Element, OpaqueElement};
 
 /// What kind of selector matching mode we should use.
@@ -68,6 +70,19 @@ impl VisitedHandlingMode {
     }
 }
 
+/// The mode to use whether we should matching rules inside @starting-style.
+/// https://drafts.csswg.org/css-transitions-2/#starting-style
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+pub enum IncludeStartingStyle {
+    /// All without rules inside @starting-style. This is for the most common case because the
+    /// primary/pseudo styles doesn't use rules inside @starting-style.
+    No,
+    /// Get the starting style. The starting style for an element as the after-change style with
+    /// @starting-style rules applied in addition. In other words, this matches all rules,
+    /// including rules inside @starting-style.
+    Yes,
+}
+
 /// Whether we need to set selector invalidation flags on elements for this
 /// match request.
 #[derive(Clone, Copy, Debug, PartialEq)]
@@ -76,6 +91,21 @@ pub enum NeedsSelectorFlags {
     Yes,
 }
 
+/// Whether we're matching in the contect of invalidation.
+#[derive(Clone, Copy, PartialEq)]
+pub enum MatchingForInvalidation {
+    No,
+    Yes,
+    YesForComparison,
+}
+
+impl MatchingForInvalidation {
+    /// Are we matching for invalidation?
+    pub fn is_for_invalidation(&self) -> bool {
+        matches!(*self, Self::Yes | Self::YesForComparison)
+    }
+}
+
 /// Which quirks mode is this document in.
 ///
 /// See: https://quirks.spec.whatwg.org/
@@ -99,6 +129,51 @@ impl QuirksMode {
     }
 }
 
+/// Whether or not this matching considered relative selector.
+#[derive(Clone, Copy, Debug, PartialEq)]
+pub enum RelativeSelectorMatchingState {
+    /// Was not considered for any relative selector.
+    None,
+    /// Relative selector was considered for a match, but the element to be
+    /// under matching would not anchor the relative selector. i.e. The
+    /// relative selector was not part of the first compound selector (in match
+    /// order).
+    Considered,
+    /// Same as above, but the relative selector was part of the first compound
+    /// selector (in match order).
+    ConsideredAnchor,
+}
+
+impl RelativeSelectorMatchingState {
+    /// Update the matching state to indicate that the relative selector matching
+    /// happened in the subject position.
+    pub fn considered_anchor(&mut self) {
+        *self = Self::ConsideredAnchor;
+    }
+
+    /// Update the matching state to indicate that the relative selector matching
+    /// happened in a non-subject position.
+    pub fn considered(&mut self) {
+        // Being considered an anchor is stronger (e.g. `:has(.a):is(:has(.b) .c)`).
+        if *self == Self::ConsideredAnchor {
+            *self = Self::ConsideredAnchor;
+        } else {
+            *self = Self::Considered;
+        }
+    }
+}
+
+/// Set of caches (And cache-likes) that speed up expensive selector matches.
+#[derive(Default)]
+pub struct SelectorCaches {
+    /// A cache to speed up nth-index-like selectors.
+    pub nth_index: NthIndexCache,
+    /// A cache to speed up relative selector matches. See module documentation.
+    pub relative_selector: RelativeSelectorCache,
+    /// A map of bloom filters to fast-reject relative selector matches.
+    pub relative_selector_filter_map: RelativeSelectorFilterMap,
+}
+
 /// Data associated with the matching process for a element.  This context is
 /// used across many selectors for an element, so it's not appropriate for
 /// transient data that applies to only a single selector.
@@ -110,8 +185,6 @@ where
     matching_mode: MatchingMode,
     /// Input with the bloom filter used to fast-reject selectors.
     pub bloom_filter: Option<&'a BloomFilter>,
-    /// A cache to speed up nth-index-like selectors.
-    pub nth_index_cache: &'a mut NthIndexCache,
     /// The element which is going to match :scope pseudo-class. It can be
     /// either one :scope element, or the scoping element.
     ///
@@ -131,6 +204,12 @@ where
     /// Controls how matching for links is handled.
     visited_handling: VisitedHandlingMode,
 
+    /// Controls if we should match rules in @starting-style.
+    pub include_starting_style: IncludeStartingStyle,
+
+    /// Whether there are any rules inside @starting-style.
+    pub has_starting_style: bool,
+
     /// The current nesting level of selectors that we're matching.
     nesting_level: usize,
 
@@ -146,10 +225,17 @@ where
 
     /// The current element we're anchoring on for evaluating the relative selector.
     current_relative_selector_anchor: Option<OpaqueElement>,
-    pub considered_relative_selector: bool,
+    pub considered_relative_selector: RelativeSelectorMatchingState,
 
     quirks_mode: QuirksMode,
     needs_selector_flags: NeedsSelectorFlags,
+
+    /// Whether we're matching in the contect of invalidation.
+    matching_for_invalidation: MatchingForInvalidation,
+
+    /// Caches to speed up expensive selector matches.
+    pub selector_caches: &'a mut SelectorCaches,
+
     classes_and_ids_case_sensitivity: CaseSensitivity,
     _impl: ::std::marker::PhantomData<Impl>,
 }
@@ -162,17 +248,20 @@ where
     pub fn new(
         matching_mode: MatchingMode,
         bloom_filter: Option<&'a BloomFilter>,
-        nth_index_cache: &'a mut NthIndexCache,
+        selector_caches: &'a mut SelectorCaches,
         quirks_mode: QuirksMode,
         needs_selector_flags: NeedsSelectorFlags,
+        matching_for_invalidation: MatchingForInvalidation,
     ) -> Self {
         Self::new_for_visited(
             matching_mode,
             bloom_filter,
-            nth_index_cache,
+            selector_caches,
             VisitedHandlingMode::AllLinksUnvisited,
+            IncludeStartingStyle::No,
             quirks_mode,
             needs_selector_flags,
+            matching_for_invalidation,
         )
     }
 
@@ -180,19 +269,23 @@ where
     pub fn new_for_visited(
         matching_mode: MatchingMode,
         bloom_filter: Option<&'a BloomFilter>,
-        nth_index_cache: &'a mut NthIndexCache,
+        selector_caches: &'a mut SelectorCaches,
         visited_handling: VisitedHandlingMode,
+        include_starting_style: IncludeStartingStyle,
         quirks_mode: QuirksMode,
         needs_selector_flags: NeedsSelectorFlags,
+        matching_for_invalidation: MatchingForInvalidation,
     ) -> Self {
         Self {
             matching_mode,
             bloom_filter,
             visited_handling,
-            nth_index_cache,
+            include_starting_style,
+            has_starting_style: false,
             quirks_mode,
             classes_and_ids_case_sensitivity: quirks_mode.classes_and_ids_case_sensitivity(),
             needs_selector_flags,
+            matching_for_invalidation,
             scope_element: None,
             current_host: None,
             nesting_level: 0,
@@ -200,7 +293,8 @@ where
             pseudo_element_matching_fn: None,
             extra_data: Default::default(),
             current_relative_selector_anchor: None,
-            considered_relative_selector: false,
+            considered_relative_selector: RelativeSelectorMatchingState::None,
+            selector_caches,
             _impl: ::std::marker::PhantomData,
         }
     }
@@ -213,7 +307,9 @@ where
         is_from_end: bool,
         selectors: &[Selector<Impl>],
     ) -> &mut NthIndexCacheInner {
-        self.nth_index_cache.get(is_of_type, is_from_end, selectors)
+        self.selector_caches
+            .nth_index
+            .get(is_of_type, is_from_end, selectors)
     }
 
     /// Whether we're matching a nested selector.
@@ -246,6 +342,36 @@ where
         self.needs_selector_flags == NeedsSelectorFlags::Yes
     }
 
+    /// Whether or not we're matching to invalidate.
+    #[inline]
+    pub fn matching_for_invalidation(&self) -> bool {
+        self.matching_for_invalidation.is_for_invalidation()
+    }
+
+    /// Whether or not we're comparing for invalidation, if we are matching for invalidation.
+    #[inline]
+    pub fn matching_for_invalidation_comparison(&self) -> Option<bool> {
+        match self.matching_for_invalidation {
+            MatchingForInvalidation::No => None,
+            MatchingForInvalidation::Yes => Some(false),
+            MatchingForInvalidation::YesForComparison => Some(true),
+        }
+    }
+
+    /// Run the given matching function for before/after invalidation comparison.
+    #[inline]
+    pub fn for_invalidation_comparison<F, R>(&mut self, f: F) -> R
+    where
+        F: FnOnce(&mut Self) -> R,
+    {
+        debug_assert!(self.matching_for_invalidation(), "Not matching for invalidation?");
+        let prev = self.matching_for_invalidation;
+        self.matching_for_invalidation = MatchingForInvalidation::YesForComparison;
+        let result = f(self);
+        self.matching_for_invalidation = prev;
+        result
+    }
+
     /// The case-sensitivity for class and ID selectors
     #[inline]
     pub fn classes_and_ids_case_sensitivity(&self) -> CaseSensitivity {
@@ -272,7 +398,7 @@ where
         F: FnOnce(&mut Self) -> R,
     {
         let old_in_negation = self.in_negation;
-        self.in_negation = true;
+        self.in_negation = !self.in_negation;
         let result = self.nest(f);
         self.in_negation = old_in_negation;
         result
@@ -329,12 +455,42 @@ where
     where
         F: FnOnce(&mut Self) -> R,
     {
-        // TODO(dshin): Nesting should be rejected at parse time.
-        let original_relative_selector_anchor = self.current_relative_selector_anchor.take();
+        debug_assert!(
+            self.current_relative_selector_anchor.is_none(),
+            "Nesting should've been rejected at parse time"
+        );
         self.current_relative_selector_anchor = Some(anchor);
         let result = self.nest(f);
-        self.current_relative_selector_anchor = original_relative_selector_anchor;
-        self.considered_relative_selector = true;
+        self.current_relative_selector_anchor = None;
+        result
+    }
+
+    /// Runs F with a deeper nesting level, with the given element as the scope.
+    #[inline]
+    pub fn nest_for_scope<F, R>(&mut self, scope: Option<OpaqueElement>, f: F) -> R
+    where
+        F: FnOnce(&mut Self) -> R,
+    {
+        let original_scope_element = self.scope_element;
+        self.scope_element = scope;
+        let result = f(self);
+        self.scope_element = original_scope_element;
+        result
+    }
+
+    /// Runs F with a deeper nesting level, with the given element as the scope, for
+    /// matching `scope-start` and/or `scope-end` conditions.
+    #[inline]
+    pub fn nest_for_scope_condition<F, R>(&mut self, scope: Option<OpaqueElement>, f: F) -> R
+    where
+        F: FnOnce(&mut Self) -> R,
+    {
+        let original_matching_mode = self.matching_mode;
+        // We may as well be matching for a pseudo-element inside `@scope`, but
+        // the scope-defining selectors wouldn't be matching them.
+        self.matching_mode = MatchingMode::Normal;
+        let result = self.nest_for_scope(scope, f);
+        self.matching_mode = original_matching_mode;
         result
     }
 
