@@ -31,7 +31,6 @@ macro_rules! apply_non_ts_list {
         $apply_macro! {
             [
                 ("-moz-table-border-nonzero", MozTableBorderNonzero, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
-                ("-moz-browser-frame", MozBrowserFrame, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS_AND_CHROME),
                 ("-moz-select-list-box", MozSelectListBox, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
                 ("link", Link, UNVISITED, _),
                 ("any-link", AnyLink, VISITED_OR_UNVISITED, _),
@@ -55,8 +54,7 @@ macro_rules! apply_non_ts_list {
                 ("fullscreen", Fullscreen, FULLSCREEN, _),
                 ("modal", Modal, MODAL, _),
                 ("-moz-topmost-modal", MozTopmostModal, TOPMOST_MODAL, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
-                ("-moz-broken", MozBroken, BROKEN, _),
-                ("-moz-loading", MozLoading, LOADING, _),
+                ("-moz-broken", MozBroken, BROKEN, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS_AND_CHROME),
                 ("-moz-has-dir-attr", MozHasDirAttr, HAS_DIR_ATTR, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
                 ("-moz-dir-attr-ltr", MozDirAttrLTR, HAS_DIR_ATTR_LTR, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
                 ("-moz-dir-attr-rtl", MozDirAttrRTL, HAS_DIR_ATTR_RTL, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
@@ -89,10 +87,17 @@ macro_rules! apply_non_ts_list {
                 ("-moz-last-node", MozLastNode, _, _),
                 ("-moz-only-whitespace", MozOnlyWhitespace, _, _),
                 ("-moz-native-anonymous", MozNativeAnonymous, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
-                ("-moz-use-shadow-tree-root", MozUseShadowTreeRoot, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
-                ("-moz-is-html", MozIsHTML, _, _),
                 ("-moz-placeholder", MozPlaceholder, _, _),
-                ("-moz-lwtheme", MozLWTheme, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS_AND_CHROME),
+
+                // NOTE(emilio): Pseudo-classes below only depend on document state, and thus
+                // conceptually they should probably be media queries instead.
+                //
+                // However that has a set of trade-offs that might not be worth making. In
+                // particular, such media queries would prevent documents that match them from
+                // sharing user-agent stylesheets with documents that don't. Also, changes between
+                // media query results are more expensive than document state changes. So for now
+                // making them pseudo-classes is probably the right trade-off.
+                ("-moz-is-html", MozIsHTML, _, PSEUDO_CLASS_ENABLED_IN_UA_SHEETS),
                 ("-moz-window-inactive", MozWindowInactive, _, _),
             ]
         }
