@@ -3,21 +3,18 @@
  * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 <%namespace name="helpers" file="/helpers.mako.rs" />
-<% from data import ALL_AXES, Keyword, Method, to_rust_ident, to_camel_case%>
-
-<% data.new_style_struct("Box",
-                         inherited=False,
-                         gecko_name="Display") %>
+<% from data import ALL_AXES %>
 
 ${helpers.predefined_type(
     "display",
     "Display",
     "computed::Display::inline()",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_specified_value="specified::Display::inline()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-display/#propdef-display",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
@@ -29,6 +26,7 @@ ${helpers.single_keyword(
     animation_value_type="none",
     enabled_in="ua",
     spec="Internal (not web-exposed)",
+    affects="layout",
 )}
 
 // An internal-only property for elements in a top layer
@@ -36,20 +34,22 @@ ${helpers.single_keyword(
 ${helpers.single_keyword(
     "-servo-top-layer",
     "none top",
-    engines="servo-2013 servo-2020",
+    engines="servo",
     animation_value_type="none",
     enabled_in="ua",
     spec="Internal (not web-exposed)",
+    affects="layout",
 )}
 
 <%helpers:single_keyword
     name="position"
-    values="static absolute relative fixed ${'sticky' if engine in ['gecko', 'servo-2013'] else ''}"
-    engines="gecko servo-2013 servo-2020"
+    values="static absolute relative fixed sticky"
+    engines="gecko servo"
     animation_value_type="discrete"
     gecko_enum_prefix="StylePositionProperty"
     spec="https://drafts.csswg.org/css-position/#position-property"
     servo_restyle_damage="rebuild_and_reflow"
+    affects="layout"
 >
 impl computed_value::T {
     pub fn is_absolutely_positioned(self) -> bool {
@@ -61,47 +61,76 @@ impl computed_value::T {
 }
 </%helpers:single_keyword>
 
+// Changes do not invalidate our element. We handle notify/invalidating
+// elements that reference anchor-name elsewhere.
+${helpers.predefined_type(
+    "anchor-name",
+    "AnchorName",
+    "computed::AnchorName::none()",
+    engines="gecko",
+    animation_value_type="discrete",
+    gecko_pref="layout.css.anchor-positioning.enabled",
+    spec="https://drafts.csswg.org/css-anchor-position-1/#propdef-anchor-name",
+    affects="",
+)}
+
+// Changes do not invalidate our element. We handle notify/invalidating
+// any affected descendants elsewhere.
+${helpers.predefined_type(
+    "anchor-scope",
+    "AnchorScope",
+    "computed::AnchorScope::none()",
+    engines="gecko",
+    animation_value_type="discrete",
+    gecko_pref="layout.css.anchor-positioning.enabled",
+    spec="https://drafts.csswg.org/css-anchor-position-1/#propdef-scope",
+    affects="",
+)}
+
 ${helpers.predefined_type(
     "float",
     "Float",
     "computed::Float::None",
-    engines="gecko servo-2013 servo-2020",
-    servo_2020_pref="layout.2020.unimplemented",
+    engines="gecko servo",
     initial_specified_value="specified::Float::None",
     spec="https://drafts.csswg.org/css-box/#propdef-float",
     animation_value_type="discrete",
     servo_restyle_damage="rebuild_and_reflow",
     gecko_ffi_name="mFloat",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "clear",
     "Clear",
     "computed::Clear::None",
-    engines="gecko servo-2013",
+    engines="gecko servo",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css2/#propdef-clear",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "vertical-align",
     "VerticalAlign",
     "computed::VerticalAlign::baseline()",
-    engines="gecko servo-2013",
+    engines="gecko servo",
     animation_value_type="ComputedValue",
     spec="https://www.w3.org/TR/CSS2/visudet.html#propdef-vertical-align",
     servo_restyle_damage = "reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "baseline-source",
     "BaselineSource",
     "computed::BaselineSource::Auto",
-    engines="gecko servo-2013",
+    engines="gecko servo",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-inline-3/#baseline-source",
     servo_restyle_damage = "reflow",
+    affects="layout",
 )}
 
 // CSS 2.1, Section 11 - Visual effects
@@ -109,11 +138,13 @@ ${helpers.predefined_type(
 ${helpers.single_keyword(
     "-servo-overflow-clip-box",
     "padding-box content-box",
-    engines="servo-2013",
+    engines="servo",
+    servo_pref="layout.legacy_layout",
     animation_value_type="none",
     enabled_in="ua",
     spec="Internal, not web-exposed, \
           may be standardized in the future (https://developer.mozilla.org/en-US/docs/Web/CSS/overflow-clip-box)",
+    affects="layout",
 )}
 
 % for direction in ["inline", "block"]:
@@ -127,6 +158,7 @@ ${helpers.single_keyword(
         animation_value_type="discrete",
         spec="Internal, may be standardized in the future: \
               https://developer.mozilla.org/en-US/docs/Web/CSS/overflow-clip-box",
+        affects="layout",
     )}
 % endfor
 
@@ -136,13 +168,13 @@ ${helpers.single_keyword(
         full_name,
         "Overflow",
         "computed::Overflow::Visible",
-        engines="gecko servo-2013 servo-2020",
+        engines="gecko servo",
         logical_group="overflow",
         logical=logical,
         animation_value_type="discrete",
         spec="https://drafts.csswg.org/css-overflow-3/#propdef-{}".format(full_name),
         servo_restyle_damage = "reflow",
-        gecko_pref="layout.css.overflow-logical.enabled" if logical else None,
+        affects="layout",
     )}
 % endfor
 
@@ -155,6 +187,7 @@ ${helpers.predefined_type(
     gecko_pref="layout.css.scroll-anchoring.enabled",
     spec="https://drafts.csswg.org/css-scroll-anchoring/#exclusion-api",
     animation_value_type="discrete",
+    affects="",
 )}
 
 <% transform_extra_prefixes = "moz:layout.css.prefixes.transforms webkit" %>
@@ -163,51 +196,52 @@ ${helpers.predefined_type(
     "transform",
     "Transform",
     "generics::transform::Transform::none()",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     extra_prefixes=transform_extra_prefixes,
     animation_value_type="ComputedValue",
     flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.csswg.org/css-transforms/#propdef-transform",
     servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
     "rotate",
     "Rotate",
     "generics::transform::Rotate::None",
-    engines="gecko servo-2013",
+    engines="gecko servo",
     animation_value_type="ComputedValue",
     boxed=True,
     flags="CAN_ANIMATE_ON_COMPOSITOR",
-    gecko_pref="layout.css.individual-transform.enabled",
     spec="https://drafts.csswg.org/css-transforms-2/#individual-transforms",
     servo_restyle_damage = "reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
     "scale",
     "Scale",
     "generics::transform::Scale::None",
-    engines="gecko servo-2013",
+    engines="gecko servo",
     animation_value_type="ComputedValue",
     boxed=True,
     flags="CAN_ANIMATE_ON_COMPOSITOR",
-    gecko_pref="layout.css.individual-transform.enabled",
     spec="https://drafts.csswg.org/css-transforms-2/#individual-transforms",
     servo_restyle_damage = "reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
     "translate",
     "Translate",
     "generics::transform::Translate::None",
-    engines="gecko servo-2013",
+    engines="gecko servo",
     animation_value_type="ComputedValue",
     boxed=True,
     flags="CAN_ANIMATE_ON_COMPOSITOR",
-    gecko_pref="layout.css.individual-transform.enabled",
     spec="https://drafts.csswg.org/css-transforms-2/#individual-transforms",
     servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 // Motion Path Module Level 1
@@ -216,11 +250,11 @@ ${helpers.predefined_type(
     "OffsetPath",
     "computed::OffsetPath::none()",
     engines="gecko",
-    animation_value_type="ComputedValue",
-    gecko_pref="layout.css.motion-path.enabled",
+    animation_value_type="motion::OffsetPath",
     flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.fxtf.org/motion-1/#offset-path-property",
-    servo_restyle_damage="reflow_out_of_flow"
+    servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 // Motion Path Module Level 1
@@ -230,10 +264,10 @@ ${helpers.predefined_type(
     "computed::LengthPercentage::zero()",
     engines="gecko",
     animation_value_type="ComputedValue",
-    gecko_pref="layout.css.motion-path.enabled",
     flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.fxtf.org/motion-1/#offset-distance-property",
-    servo_restyle_damage="reflow_out_of_flow"
+    servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 // Motion Path Module Level 1
@@ -243,10 +277,10 @@ ${helpers.predefined_type(
     "computed::OffsetRotate::auto()",
     engines="gecko",
     animation_value_type="ComputedValue",
-    gecko_pref="layout.css.motion-path.enabled",
     flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.fxtf.org/motion-1/#offset-rotate-property",
-    servo_restyle_damage="reflow_out_of_flow"
+    servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 // Motion Path Module Level 1
@@ -256,24 +290,25 @@ ${helpers.predefined_type(
     "computed::PositionOrAuto::auto()",
     engines="gecko",
     animation_value_type="ComputedValue",
-    gecko_pref="layout.css.motion-path.enabled",
     flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.fxtf.org/motion-1/#offset-anchor-property",
     servo_restyle_damage="reflow_out_of_flow",
-    boxed=True
+    boxed=True,
+    affects="overflow",
 )}
 
 // Motion Path Module Level 1
 ${helpers.predefined_type(
     "offset-position",
     "OffsetPosition",
-    "computed::OffsetPosition::auto()",
+    "computed::OffsetPosition::normal()",
     engines="gecko",
     animation_value_type="ComputedValue",
-    gecko_pref="layout.css.motion-path-offset-position.enabled",
+    flags="CAN_ANIMATE_ON_COMPOSITOR",
     spec="https://drafts.fxtf.org/motion-1/#offset-position-property",
     servo_restyle_damage="reflow_out_of_flow",
-    boxed=True
+    boxed=True,
+    affects="overflow",
 )}
 
 // CSSOM View Module
@@ -285,6 +320,7 @@ ${helpers.single_keyword(
     spec="https://drafts.csswg.org/cssom-view/#propdef-scroll-behavior",
     animation_value_type="discrete",
     gecko_enum_prefix="StyleScrollBehavior",
+    affects="",
 )}
 
 ${helpers.predefined_type(
@@ -294,6 +330,7 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-scroll-snap-1/#scroll-snap-align",
     animation_value_type="discrete",
+    affects="paint",
 )}
 
 ${helpers.predefined_type(
@@ -303,6 +340,7 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-scroll-snap-1/#scroll-snap-type",
     animation_value_type="discrete",
+    affects="paint",
 )}
 
 ${helpers.predefined_type(
@@ -312,6 +350,7 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-scroll-snap-1/#scroll-snap-stop",
     animation_value_type="discrete",
+    affects="paint",
 )}
 
 % for (axis, logical) in ALL_AXES:
@@ -325,6 +364,7 @@ ${helpers.predefined_type(
         gecko_pref="layout.css.overscroll-behavior.enabled",
         spec="https://wicg.github.io/overscroll-behavior/#overscroll-behavior-properties",
         animation_value_type="discrete",
+        affects="paint",
     )}
 % endfor
 
@@ -337,6 +377,7 @@ ${helpers.single_keyword(
     spec="https://drafts.fxtf.org/compositing/#isolation",
     gecko_enum_prefix="StyleIsolation",
     animation_value_type="discrete",
+    affects="paint",
 )}
 
 ${helpers.predefined_type(
@@ -346,6 +387,7 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-break/#propdef-break-after",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -355,6 +397,7 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-break/#propdef-break-before",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -364,6 +407,7 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-break/#propdef-break-inside",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 // CSS Basic User Interface Module Level 3
@@ -376,74 +420,80 @@ ${helpers.predefined_type(
     animation_value_type="discrete",
     gecko_ffi_name="mResize",
     spec="https://drafts.csswg.org/css-ui/#propdef-resize",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "perspective",
     "Perspective",
     "computed::Perspective::none()",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     gecko_ffi_name="mChildPerspective",
     spec="https://drafts.csswg.org/css-transforms/#perspective",
     extra_prefixes=transform_extra_prefixes,
     animation_value_type="AnimatedPerspective",
     servo_restyle_damage = "reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
     "perspective-origin",
     "Position",
     "computed::position::Position::center()",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     boxed=True,
     extra_prefixes=transform_extra_prefixes,
     spec="https://drafts.csswg.org/css-transforms-2/#perspective-origin-property",
     animation_value_type="ComputedValue",
-    servo_restyle_damage="reflow_out_of_flow"
+    servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.single_keyword(
     "backface-visibility",
     "visible hidden",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     gecko_enum_prefix="StyleBackfaceVisibility",
     spec="https://drafts.csswg.org/css-transforms/#backface-visibility-property",
     extra_prefixes=transform_extra_prefixes,
     animation_value_type="discrete",
+    affects="paint",
 )}
 
-${helpers.single_keyword(
+${helpers.predefined_type(
     "transform-box",
-    "border-box fill-box view-box",
+    "TransformBox",
+    "computed::TransformBox::ViewBox",
     engines="gecko",
-    gecko_enum_prefix="StyleGeometryBox",
     spec="https://drafts.csswg.org/css-transforms/#transform-box",
-    gecko_inexhaustive="True",
     animation_value_type="discrete",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
     "transform-style",
     "TransformStyle",
     "computed::TransformStyle::Flat",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     spec="https://drafts.csswg.org/css-transforms-2/#transform-style-property",
     extra_prefixes=transform_extra_prefixes,
     animation_value_type="discrete",
     servo_restyle_damage = "reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
     "transform-origin",
     "TransformOrigin",
     "computed::TransformOrigin::initial_value()",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     animation_value_type="ComputedValue",
     extra_prefixes=transform_extra_prefixes,
     gecko_ffi_name="mTransformOrigin",
     boxed=True,
     spec="https://drafts.csswg.org/css-transforms/#transform-origin-property",
     servo_restyle_damage="reflow_out_of_flow",
+    affects="overflow",
 )}
 
 ${helpers.predefined_type(
@@ -453,6 +503,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="none",
     spec="https://drafts.csswg.org/css-contain/#contain-property",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -462,29 +513,30 @@ ${helpers.predefined_type(
     engines="gecko",
     spec="https://drafts.csswg.org/css-contain/#content-visibility",
     gecko_pref="layout.css.content-visibility.enabled",
-    animation_value_type="none",
+    animation_value_type="ComputedValue",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "container-type",
     "ContainerType",
     "computed::ContainerType::Normal",
-    engines="gecko",
+    engines="gecko servo",
     animation_value_type="none",
-    enabled_in="ua",
-    gecko_pref="layout.css.container-queries.enabled",
+    servo_pref="layout.container-queries.enabled",
     spec="https://drafts.csswg.org/css-contain-3/#container-type",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "container-name",
     "ContainerName",
     "computed::ContainerName::none()",
-    engines="gecko",
+    engines="gecko servo",
     animation_value_type="none",
-    enabled_in="ua",
-    gecko_pref="layout.css.container-queries.enabled",
+    servo_pref="layout.container-queries.enabled",
     spec="https://drafts.csswg.org/css-contain-3/#container-name",
+    affects="",
 )}
 
 ${helpers.predefined_type(
@@ -496,6 +548,7 @@ ${helpers.predefined_type(
     spec="https://drafts.csswg.org/css-ui-4/#propdef-appearance",
     animation_value_type="discrete",
     gecko_ffi_name="mAppearance",
+    affects="paint",
 )}
 
 // The inherent widget type of an element, selected by specifying
@@ -509,6 +562,7 @@ ${helpers.predefined_type(
     spec="Internal (not web-exposed)",
     enabled_in="chrome",
     gecko_ffi_name="mDefaultAppearance",
+    affects="paint",
 )}
 
 ${helpers.single_keyword(
@@ -519,6 +573,7 @@ ${helpers.single_keyword(
     gecko_enum_prefix="StyleOrient",
     spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-orient)",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -528,6 +583,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="none",
     spec="https://drafts.csswg.org/css-will-change/#will-change",
+    affects="layout",
 )}
 
 // The spec issue for the parse_method: https://github.com/w3c/csswg-drafts/issues/4102.
@@ -538,6 +594,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="ComputedValue",
     spec="https://drafts.csswg.org/css-shapes/#shape-image-threshold-property",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -547,6 +604,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="NonNegativeLengthPercentage",
     spec="https://drafts.csswg.org/css-shapes/#shape-margin-property",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -556,6 +614,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="basic_shape::ShapeOutside",
     spec="https://drafts.csswg.org/css-shapes/#shape-outside-property",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -565,6 +624,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="discrete",
     spec="https://compat.spec.whatwg.org/#touch-action",
+    affects="paint",
 )}
 
 ${helpers.predefined_type(
@@ -574,6 +634,7 @@ ${helpers.predefined_type(
     engines="gecko",
     animation_value_type="ComputedValue",
     spec="https://drafts.csswg.org/css-overflow-3/#line-clamp",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -581,7 +642,19 @@ ${helpers.predefined_type(
     "ScrollbarGutter",
     "computed::ScrollbarGutter::AUTO",
     engines="gecko",
-    gecko_pref="layout.css.scrollbar-gutter.enabled",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-overflow-3/#scrollbar-gutter-property",
+    affects="layout",
+)}
+
+${helpers.predefined_type(
+    "zoom",
+    "Zoom",
+    "computed::box_::Zoom::ONE",
+    engines="gecko",
+    animation_value_type="Number",
+    spec="Non-standard (https://github.com/atanassov/css-zoom/ is the closest)",
+    gecko_pref="layout.css.zoom.enabled",
+    affects="layout",
+    enabled_in="chrome",
 )}
