@@ -4,56 +4,51 @@
 
 <%namespace name="helpers" file="/helpers.mako.rs" />
 
-<% data.new_style_struct("Column", inherited=False) %>
-
 ${helpers.predefined_type(
     "column-width",
     "length::NonNegativeLengthOrAuto",
     "computed::length::NonNegativeLengthOrAuto::auto()",
-    engines="gecko servo-2013 servo-2020",
-    servo_2020_pref="layout.2020.unimplemented",
+    engines="gecko servo",
     initial_specified_value="specified::length::NonNegativeLengthOrAuto::auto()",
-    extra_prefixes="moz:layout.css.prefixes.columns",
     animation_value_type="NonNegativeLengthOrAuto",
-    servo_2013_pref="layout.columns.enabled",
+    servo_pref="layout.columns.enabled",
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-width",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "column-count",
     "ColumnCount",
-    "computed::ColumnCount::auto()",
-    engines="gecko servo-2013 servo-2020",
-    servo_2020_pref="layout.2020.unimplemented",
-    initial_specified_value="specified::ColumnCount::auto()",
-    servo_2013_pref="layout.columns.enabled",
+    "computed::ColumnCount::Auto",
+    engines="gecko servo",
+    initial_specified_value="specified::ColumnCount::Auto",
+    servo_pref="layout.columns.enabled",
     animation_value_type="AnimatedColumnCount",
-    extra_prefixes="moz:layout.css.prefixes.columns",
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-count",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
     "column-fill",
     "balance auto",
     engines="gecko",
-    extra_prefixes="moz:layout.css.prefixes.columns",
     animation_value_type="discrete",
     gecko_enum_prefix="StyleColumnFill",
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-fill",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "column-rule-width",
     "BorderSideWidth",
-    "crate::values::computed::NonNegativeLength::new(3.)",
+    "app_units::Au::from_px(3)",
     engines="gecko",
-    initial_specified_value="specified::BorderSideWidth::Medium",
-    computed_type="crate::values::computed::NonNegativeLength",
+    initial_specified_value="specified::BorderSideWidth::medium()",
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-rule-width",
     animation_value_type="NonNegativeLength",
-    extra_prefixes="moz:layout.css.prefixes.columns",
+    affects="layout",
 )}
 
 // https://drafts.csswg.org/css-multicol-1/#crc
@@ -64,19 +59,20 @@ ${helpers.predefined_type(
     engines="gecko",
     initial_specified_value="specified::Color::currentcolor()",
     animation_value_type="AnimatedColor",
-    extra_prefixes="moz:layout.css.prefixes.columns",
     ignored_when_colors_disabled=True,
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-rule-color",
+    affects="paint",
 )}
 
 ${helpers.single_keyword(
     "column-span",
     "none all",
-    engines="gecko",
+    engines="gecko servo",
+    servo_pref="layout.columns.enabled",
     animation_value_type="discrete",
     gecko_enum_prefix="StyleColumnSpan",
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-span",
-    extra_prefixes="moz:layout.css.prefixes.columns",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -84,9 +80,8 @@ ${helpers.predefined_type(
     "BorderStyle",
     "computed::BorderStyle::None",
     engines="gecko",
-    needs_context=False,
     initial_specified_value="specified::BorderStyle::None",
-    extra_prefixes="moz:layout.css.prefixes.columns",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-multicol/#propdef-column-rule-style",
+    affects="paint",
 )}
