@@ -3,29 +3,29 @@
  * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 <%namespace name="helpers" file="/helpers.mako.rs" />
-<% from data import Method, to_camel_case, to_rust_ident, to_camel_case_lower, SYSTEM_FONT_LONGHANDS %>
-
-<% data.new_style_struct("Font", inherited=True) %>
+<% from data import SYSTEM_FONT_LONGHANDS %>
 
 ${helpers.predefined_type(
     "font-family",
     "FontFamily",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_value="computed::FontFamily::serif()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-family",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "font-style",
     "FontStyle",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_value="computed::FontStyle::normal()",
     initial_specified_value="specified::FontStyle::normal()",
     animation_value_type="FontStyle",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-style",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 <% font_variant_caps_custom_consts= { "small-caps": "SMALLCAPS",
@@ -37,7 +37,7 @@ ${helpers.predefined_type(
 ${helpers.single_keyword(
     "font-variant-caps",
     "normal small-caps",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     extra_gecko_values="all-small-caps petite-caps all-petite-caps unicase titling-caps",
     gecko_constant_prefix="NS_FONT_VARIANT_CAPS",
     gecko_ffi_name="mFont.variantCaps",
@@ -45,29 +45,32 @@ ${helpers.single_keyword(
     custom_consts=font_variant_caps_custom_consts,
     animation_value_type="discrete",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "font-weight",
     "FontWeight",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_value="computed::FontWeight::normal()",
     initial_specified_value="specified::FontWeight::normal()",
     animation_value_type="Number",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-weight",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "font-size",
     "FontSize",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_value="computed::FontSize::medium()",
     initial_specified_value="specified::FontSize::medium()",
     animation_value_type="NonNegativeLength",
     allow_quirks="Yes",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-size",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -78,6 +81,7 @@ ${helpers.predefined_type(
     initial_specified_value="specified::FontSizeAdjust::None",
     animation_value_type="FontSizeAdjust",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-size-adjust",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -89,6 +93,7 @@ ${helpers.predefined_type(
     gecko_ffi_name="mFont.synthesisWeight",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts-4/#font-synthesis-weight",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -100,6 +105,7 @@ ${helpers.predefined_type(
     gecko_ffi_name="mFont.synthesisStyle",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts-4/#font-synthesis-style",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -111,17 +117,31 @@ ${helpers.predefined_type(
     gecko_ffi_name="mFont.synthesisSmallCaps",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts-4/#font-synthesis-small-caps",
+    affects="layout",
+)}
+
+${helpers.predefined_type(
+    "font-synthesis-position",
+    "FontSynthesis",
+    engines="gecko",
+    initial_value="computed::FontSynthesis::Auto",
+    initial_specified_value="specified::FontSynthesis::Auto",
+    gecko_ffi_name="mFont.synthesisPosition",
+    animation_value_type="discrete",
+    spec="https://drafts.csswg.org/css-fonts-4/#font-synthesis-position",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
     "font-stretch",
     "FontStretch",
-    engines="gecko servo-2013 servo-2020",
+    engines="gecko servo",
     initial_value="computed::FontStretch::hundred()",
     initial_specified_value="specified::FontStretch::normal()",
     animation_value_type="Percentage",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-stretch",
     servo_restyle_damage="rebuild_and_reflow",
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
@@ -132,6 +152,7 @@ ${helpers.single_keyword(
     gecko_constant_prefix="NS_FONT_KERNING",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-kerning",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -142,6 +163,7 @@ ${helpers.predefined_type(
     initial_specified_value="specified::FontVariantAlternates::default()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-variant-alternates",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -152,6 +174,7 @@ ${helpers.predefined_type(
     initial_specified_value="specified::FontVariantEastAsian::empty()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-variant-east-asian",
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
@@ -164,6 +187,7 @@ ${helpers.single_keyword(
     gecko_ffi_name="mFont.variantEmoji",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-variant-emoji",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -174,6 +198,7 @@ ${helpers.predefined_type(
     initial_specified_value="specified::FontVariantLigatures::empty()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-variant-ligatures",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -184,6 +209,7 @@ ${helpers.predefined_type(
     initial_specified_value="specified::FontVariantNumeric::empty()",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-variant-numeric",
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
@@ -194,6 +220,7 @@ ${helpers.single_keyword(
     gecko_constant_prefix="NS_FONT_VARIANT_POSITION",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-variant-position",
     animation_value_type="discrete",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -205,6 +232,7 @@ ${helpers.predefined_type(
     extra_prefixes="moz:layout.css.prefixes.font-features",
     animation_value_type="discrete",
     spec="https://drafts.csswg.org/css-fonts/#propdef-font-feature-settings",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -216,7 +244,8 @@ ${helpers.predefined_type(
     initial_value="computed::FontVariationSettings::normal()",
     initial_specified_value="specified::FontVariationSettings::normal()",
     animation_value_type="ComputedValue",
-    spec="https://drafts.csswg.org/css-fonts-4/#propdef-font-variation-settings"
+    spec="https://drafts.csswg.org/css-fonts-4/#propdef-font-variation-settings",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -228,6 +257,7 @@ ${helpers.predefined_type(
     animation_value_type="discrete",
     extra_prefixes="moz:layout.css.prefixes.font-features",
     spec="https://drafts.csswg.org/css-fonts-3/#propdef-font-language-override",
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
@@ -240,6 +270,7 @@ ${helpers.single_keyword(
     gecko_constant_prefix="NS_FONT_OPTICAL_SIZING",
     animation_value_type="discrete",
     spec="https://www.w3.org/TR/css-fonts-4/#font-optical-sizing-def",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -252,6 +283,7 @@ ${helpers.predefined_type(
     gecko_pref="layout.css.font-palette.enabled",
     has_effect_on_gecko_scrollbars=False,
     spec="https://drafts.csswg.org/css-fonts/#font-palette-prop",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -263,18 +295,7 @@ ${helpers.predefined_type(
     enabled_in="",
     has_effect_on_gecko_scrollbars=False,
     spec="Internal (not web-exposed)",
-)}
-
-${helpers.predefined_type(
-    "-moz-script-size-multiplier",
-    "MozScriptSizeMultiplier",
-    engines="gecko",
-    initial_value="computed::MozScriptSizeMultiplier::get_initial_value()",
-    animation_value_type="none",
-    gecko_ffi_name="mScriptSizeMultiplier",
-    enabled_in="",
-    has_effect_on_gecko_scrollbars=False,
-    spec="Internal (not web-exposed)",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -282,11 +303,9 @@ ${helpers.predefined_type(
     "MathDepth",
     "0",
     engines="gecko",
-    gecko_pref="layout.css.math-depth.enabled",
-    has_effect_on_gecko_scrollbars=False,
     animation_value_type="none",
-    enabled_in="ua",
     spec="https://mathml-refresh.github.io/mathml-core/#the-math-script-level-property",
+    affects="",
 )}
 
 ${helpers.single_keyword(
@@ -294,12 +313,10 @@ ${helpers.single_keyword(
     "normal compact",
     engines="gecko",
     gecko_enum_prefix="StyleMathStyle",
-    gecko_pref="layout.css.math-style.enabled",
     spec="https://mathml-refresh.github.io/mathml-core/#the-math-style-property",
-    has_effect_on_gecko_scrollbars=False,
     animation_value_type="none",
-    enabled_in="ua",
     needs_conversion=True,
+    affects="layout",
 )}
 
 ${helpers.single_keyword(
@@ -316,18 +333,7 @@ ${helpers.single_keyword(
     enabled_in="",
     has_effect_on_gecko_scrollbars=False,
     needs_conversion=True,
-)}
-
-${helpers.predefined_type(
-    "-moz-script-min-size",
-    "MozScriptMinSize",
-    "specified::MozScriptMinSize::get_initial_value()",
-    engines="gecko",
-    animation_value_type="none",
-    enabled_in="",
-    has_effect_on_gecko_scrollbars=False,
-    gecko_ffi_name="mScriptMinSize",
-    spec="Internal (not web-exposed)",
+    affects="layout",
 )}
 
 ${helpers.predefined_type(
@@ -339,6 +345,18 @@ ${helpers.predefined_type(
     enabled_in="",
     has_effect_on_gecko_scrollbars=False,
     spec="Internal (not web-exposed)",
+    affects="layout",
+)}
+
+${helpers.predefined_type(
+    "line-height",
+    "LineHeight",
+    "computed::LineHeight::normal()",
+    engines="gecko servo",
+    animation_value_type="LineHeight",
+    spec="https://drafts.csswg.org/css2/visudet.html#propdef-line-height",
+    servo_restyle_damage="reflow",
+    affects="layout",
 )}
 
 % if engine == "gecko":
@@ -456,25 +474,17 @@ pub mod system_font {
 
 ${helpers.single_keyword(
     "-moz-osx-font-smoothing",
-    "auto grayscale",
+    "auto grayscale subpixel-antialiased",
     engines="gecko",
     gecko_constant_prefix="NS_FONT_SMOOTHING",
     gecko_ffi_name="mFont.smoothing",
     gecko_pref="layout.css.osx-font-smoothing.enabled",
+    aliases="-webkit-font-smoothing:layout.css.osx-font-smoothing.enabled",
+    gecko_aliases="antialiased=grayscale",
     has_effect_on_gecko_scrollbars=False,
     spec="Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/font-smooth)",
     animation_value_type="discrete",
-)}
-
-${helpers.predefined_type(
-    "-moz-font-smoothing-background-color",
-    "color::MozFontSmoothingBackgroundColor",
-    "computed::color::MozFontSmoothingBackgroundColor::transparent()",
-    engines="gecko",
-    animation_value_type="none",
-    gecko_ffi_name="mFont.fontSmoothingBackgroundColor",
-    enabled_in="chrome",
-    spec="None (Nonstandard internal property)",
+    affects="paint",
 )}
 
 ${helpers.predefined_type(
@@ -485,4 +495,5 @@ ${helpers.predefined_type(
     animation_value_type="none",
     enabled_in="ua",
     spec="Nonstandard (Internal-only)",
+    affects="layout",
 )}
