@@ -5,12 +5,12 @@
 //! A data structure to efficiently index structs containing selectors by local
 //! name, ids and hash.
 
-use crate::applicable_declarations::ApplicableDeclarationList;
+use crate::applicable_declarations::{ApplicableDeclarationList, ScopeProximity};
 use crate::context::QuirksMode;
 use crate::dom::TElement;
 use crate::rule_tree::CascadeLevel;
 use crate::selector_parser::SelectorImpl;
-use crate::stylist::{CascadeData, ContainerConditionId, Rule, Stylist};
+use crate::stylist::{CascadeData, ContainerConditionId, Rule, ScopeConditionId, Stylist};
 use crate::AllocErr;
 use crate::{Atom, LocalName, Namespace, ShrinkIfNeeded, WeakAtom};
 use dom::ElementState;
@@ -34,6 +34,9 @@ impl Default for PrecomputedHasher {
     }
 }
 
+/// A vector of relevant attributes, that can be useful for revalidation.
+pub type RelevantAttributes = thin_vec::ThinVec<LocalName>;
+
 /// This is a set of pseudo-classes that are both relatively-rare (they don't
 /// affect most elements by default) and likely or known to have global rules
 /// (in e.g., the UA sheets).
@@ -128,8 +131,6 @@ pub struct SelectorMap<T: 'static> {
     pub rare_pseudo_classes: SmallVec<[T; 1]>,
     /// All other rules.
     pub other: SmallVec<[T; 1]>,
-    /// Whether we should bucket by attribute names.
-    bucket_attributes: bool,
     /// The number of entries in this map.
     pub count: usize,
 }
@@ -153,19 +154,10 @@ impl<T> SelectorMap<T> {
             namespace_hash: HashMap::default(),
             rare_pseudo_classes: SmallVec::new(),
             other: SmallVec::new(),
-            bucket_attributes: static_prefs::pref!("layout.css.bucket-attribute-names.enabled"),
             count: 0,
         }
     }
 
-    /// Trivially constructs an empty `SelectorMap`, with attribute bucketing
-    /// explicitly disabled.
-    pub fn new_without_attribute_bucketing() -> Self {
-        let mut ret = Self::new();
-        ret.bucket_attributes = false;
-        ret
-    }
-
     /// Shrink the capacity of the map if needed.
     pub fn shrink_if_needed(&mut self) {
         self.id_hash.shrink_if_needed();
@@ -262,21 +254,19 @@ impl SelectorMap<Rule> {
             }
         });
 
-        if self.bucket_attributes {
-            rule_hash_target.each_attr_name(|name| {
-                if let Some(rules) = self.attribute_hash.get(name) {
-                    SelectorMap::get_matching_rules(
-                        element,
-                        rules,
-                        matching_rules_list,
-                        matching_context,
-                        cascade_level,
-                        cascade_data,
-                        stylist,
-                    )
-                }
-            });
-        }
+        rule_hash_target.each_attr_name(|name| {
+            if let Some(rules) = self.attribute_hash.get(name) {
+                SelectorMap::get_matching_rules(
+                    element,
+                    rules,
+                    matching_rules_list,
+                    matching_context,
+                    cascade_level,
+                    cascade_data,
+                    stylist,
+                )
+            }
+        });
 
         if let Some(rules) = self.local_name_hash.get(rule_hash_target.local_name()) {
             SelectorMap::get_matching_rules(
@@ -340,7 +330,17 @@ impl SelectorMap<Rule> {
     ) where
         E: TElement,
     {
-        for rule in rules {
+        fn add_rule_if_matching<E: TElement>(
+            element: E,
+            scope_proximity: ScopeProximity,
+            rule: &Rule,
+            matching_rules: &mut ApplicableDeclarationList,
+            matching_context: &mut MatchingContext<E::Impl>,
+            cascade_level: CascadeLevel,
+            cascade_data: &CascadeData,
+            stylist: &Stylist,
+            include_starting_style: bool,
+        ) {
             if !matches_selector(
                 &rule.selector,
                 0,
@@ -348,7 +348,7 @@ impl SelectorMap<Rule> {
                 &element,
                 matching_context,
             ) {
-                continue;
+                return;
             }
 
             if rule.container_condition_id != ContainerConditionId::none() {
@@ -358,11 +358,78 @@ impl SelectorMap<Rule> {
                     element,
                     matching_context,
                 ) {
-                    continue;
+                    return;
                 }
             }
 
-            matching_rules.push(rule.to_applicable_declaration_block(cascade_level, cascade_data));
+            if rule.is_starting_style {
+                // Set this flag if there are any rules inside @starting-style. This flag is for
+                // optimization to avoid any redundant resolution of starting style if the author
+                // doesn't specify for this element.
+                matching_context.has_starting_style = true;
+
+                if !include_starting_style {
+                    return;
+                }
+            }
+
+            matching_rules.push(rule.to_applicable_declaration_block(
+                cascade_level,
+                cascade_data,
+                scope_proximity,
+            ));
+        }
+
+        use selectors::matching::IncludeStartingStyle;
+
+        let include_starting_style = matches!(
+            matching_context.include_starting_style,
+            IncludeStartingStyle::Yes
+        );
+        for rule in rules {
+            if rule.scope_condition_id == ScopeConditionId::none() {
+                add_rule_if_matching(
+                    element,
+                    ScopeProximity::infinity(),
+                    rule,
+                    matching_rules,
+                    matching_context,
+                    cascade_level,
+                    cascade_data,
+                    stylist,
+                    include_starting_style,
+                );
+            } else {
+                // First element contains the closest matching scope, but the selector may match
+                // a scope root that is further out (e.g. `.foo > .foo .bar > .baz` for `scope-start`
+                // selector `.foo` and inner selector `.bar .baz`). This requires returning all
+                // potential scopes.
+                let scopes = cascade_data.scope_condition_matches(
+                    rule.scope_condition_id,
+                    stylist,
+                    element,
+                    matching_context,
+                );
+
+                // We had to gather scope roots first, since the scope element must change before the rule's
+                // selector is tested. Candidates are sorted in proximity.
+                for candidate in scopes {
+                    matching_context.nest_for_scope(Some(candidate.root), |context| {
+                        add_rule_if_matching(
+                            element,
+                            candidate.proximity,
+                            rule,
+                            matching_rules,
+                            context,
+                            cascade_level,
+                            cascade_data,
+                            stylist,
+                            include_starting_style,
+                        )
+                    });
+                    // Given rule may match with a scope root of further proximity - Can't just break here.
+                }
+            }
         }
     }
 }
@@ -432,11 +499,7 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
 
         let bucket = {
             let mut disjoint_buckets = SmallVec::new();
-            let bucket = find_bucket(
-                entry.selector(),
-                &mut disjoint_buckets,
-                self.bucket_attributes,
-            );
+            let bucket = find_bucket(entry.selector(), &mut disjoint_buckets);
 
             // See if inserting this selector in multiple entries in the
             // selector map would be worth it. Consider a case like:
@@ -481,12 +544,24 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
     ///
     /// FIXME(bholley) This overlaps with SelectorMap<Rule>::get_all_matching_rules,
     /// but that function is extremely hot and I'd rather not rearrange it.
-    pub fn lookup<'a, E, F>(&'a self, element: E, quirks_mode: QuirksMode, f: F) -> bool
+    pub fn lookup<'a, E, F>(
+        &'a self,
+        element: E,
+        quirks_mode: QuirksMode,
+        relevant_attributes: Option<&mut RelevantAttributes>,
+        f: F,
+    ) -> bool
     where
         E: TElement,
         F: FnMut(&'a T) -> bool,
     {
-        self.lookup_with_state(element, element.state(), quirks_mode, f)
+        self.lookup_with_state(
+            element,
+            element.state(),
+            quirks_mode,
+            relevant_attributes,
+            f,
+        )
     }
 
     #[inline]
@@ -495,6 +570,7 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
         element: E,
         element_state: ElementState,
         quirks_mode: QuirksMode,
+        mut relevant_attributes: Option<&mut RelevantAttributes>,
         mut f: F,
     ) -> bool
     where
@@ -538,24 +614,25 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
             return false;
         }
 
-        if self.bucket_attributes {
-            element.each_attr_name(|name| {
-                if done {
-                    return;
+        element.each_attr_name(|name| {
+            if done {
+                return;
+            }
+            if let Some(v) = self.attribute_hash.get(name) {
+                if let Some(ref mut relevant_attributes) = relevant_attributes {
+                    relevant_attributes.push(name.clone());
                 }
-                if let Some(v) = self.attribute_hash.get(name) {
-                    for entry in v.iter() {
-                        if !f(&entry) {
-                            done = true;
-                            return;
-                        }
+                for entry in v.iter() {
+                    if !f(&entry) {
+                        done = true;
+                        return;
                     }
                 }
-            });
-
-            if done {
-                return false;
             }
+        });
+
+        if done {
+            return false;
         }
 
         if let Some(v) = self.local_name_hash.get(element.local_name()) {
@@ -617,6 +694,7 @@ impl<T: SelectorMapEntry> SelectorMap<T> {
             element,
             element.state() | additional_states,
             quirks_mode,
+            /* relevant_attributes = */ None,
             |entry| f(entry),
         ) {
             return false;
@@ -697,26 +775,23 @@ type DisjointBuckets<'a> = SmallVec<[Bucket<'a>; 5]>;
 fn specific_bucket_for<'a>(
     component: &'a Component<SelectorImpl>,
     disjoint_buckets: &mut DisjointBuckets<'a>,
-    bucket_attributes: bool,
 ) -> Bucket<'a> {
     match *component {
         Component::Root => Bucket::Root,
         Component::ID(ref id) => Bucket::ID(id),
         Component::Class(ref class) => Bucket::Class(class),
-        Component::AttributeInNoNamespace { ref local_name, .. } if bucket_attributes => {
-            Bucket::Attribute {
-                name: local_name,
-                lower_name: local_name,
-            }
+        Component::AttributeInNoNamespace { ref local_name, .. } => Bucket::Attribute {
+            name: local_name,
+            lower_name: local_name,
         },
         Component::AttributeInNoNamespaceExists {
             ref local_name,
             ref local_name_lower,
-        } if bucket_attributes => Bucket::Attribute {
+        } => Bucket::Attribute {
             name: local_name,
             lower_name: local_name_lower,
         },
-        Component::AttributeOther(ref selector) if bucket_attributes => Bucket::Attribute {
+        Component::AttributeOther(ref selector) => Bucket::Attribute {
             name: &selector.local_name,
             lower_name: &selector.local_name_lower,
         },
@@ -745,18 +820,14 @@ fn specific_bucket_for<'a>(
         //
         // So inserting `span` in the rule hash makes sense since we want to
         // match the slotted <span>.
-        Component::Slotted(ref selector) => {
-            find_bucket(selector.iter(), disjoint_buckets, bucket_attributes)
-        },
-        Component::Host(Some(ref selector)) => {
-            find_bucket(selector.iter(), disjoint_buckets, bucket_attributes)
-        },
+        Component::Slotted(ref selector) => find_bucket(selector.iter(), disjoint_buckets),
+        Component::Host(Some(ref selector)) => find_bucket(selector.iter(), disjoint_buckets),
         Component::Is(ref list) | Component::Where(ref list) => {
             if list.len() == 1 {
-                find_bucket(list[0].iter(), disjoint_buckets, bucket_attributes)
+                find_bucket(list.slice()[0].iter(), disjoint_buckets)
             } else {
-                for selector in &**list {
-                    let bucket = find_bucket(selector.iter(), disjoint_buckets, bucket_attributes);
+                for selector in list.slice() {
+                    let bucket = find_bucket(selector.iter(), disjoint_buckets);
                     disjoint_buckets.push(bucket);
                 }
                 Bucket::Universal
@@ -782,13 +853,12 @@ fn specific_bucket_for<'a>(
 fn find_bucket<'a>(
     mut iter: SelectorIter<'a, SelectorImpl>,
     disjoint_buckets: &mut DisjointBuckets<'a>,
-    bucket_attributes: bool,
 ) -> Bucket<'a> {
     let mut current_bucket = Bucket::Universal;
 
     loop {
         for ss in &mut iter {
-            let new_bucket = specific_bucket_for(ss, disjoint_buckets, bucket_attributes);
+            let new_bucket = specific_bucket_for(ss, disjoint_buckets);
             // NOTE: When presented with the choice of multiple specific selectors, use the
             // rightmost, on the assumption that that's less common, see bug 1829540.
             if new_bucket.more_or_equally_specific_than(&current_bucket) {
