@@ -7,6 +7,7 @@
 use app_units::Au;
 use cssparser::ToCss as CssparserToCss;
 use cssparser::{serialize_string, ParseError, Parser, Token, UnicodeRange};
+use nsstring::nsCString;
 use servo_arc::Arc;
 use std::fmt::{self, Write};
 
@@ -44,7 +45,7 @@ use std::fmt::{self, Write};
 /// * `#[css(represents_keyword)]` can be used on bool fields in order to
 ///   serialize the field name if the field is true, or nothing otherwise.  It
 ///   also collects those keywords for `SpecifiedValueInfo`.
-/// * `#[css(bitflags(single="", mixed="", validate="", overlapping_bits)]` can
+/// * `#[css(bitflags(single="", mixed="", validate_mixed="", overlapping_bits)]` can
 ///   be used to derive parse / serialize / etc on bitflags. The rules for parsing
 ///   bitflags are the following:
 ///
@@ -66,8 +67,10 @@ use std::fmt::{self, Write};
 ///
 ///       But `bar baz` will be valid, as they don't share bits, and so would
 ///       `foo` with any other flag, or `bazz` on its own.
-///    * `overlapping_bits` enables some tracking during serialization of mixed
-///       flags to avoid serializing variants that can subsume other variants.
+///    * `validate_mixed` can be used to reject invalid mixed combinations, and also to simplify
+///      the type or add default ones if needed.
+///    * `overlapping_bits` enables some tracking during serialization of mixed flags to avoid
+///       serializing variants that can subsume other variants.
 ///       In the example above, you could do:
 ///         mixed="foo,bazz,bar,baz", overlapping_bits
 ///       to ensure that if bazz is serialized, bar and baz aren't, even though
@@ -91,6 +94,16 @@ pub trait ToCss {
         self.to_css(&mut CssWriter::new(&mut s)).unwrap();
         s
     }
+
+    /// Serialize `self` in CSS syntax and return a nsCString.
+    ///
+    /// (This is a convenience wrapper for `to_css` and probably should not be overridden.)
+    #[inline]
+    fn to_css_nscstring(&self) -> nsCString {
+        let mut s = nsCString::new();
+        self.to_css(&mut CssWriter::new(&mut s)).unwrap();
+        s
+    }
 }
 
 impl<'a, T> ToCss for &'a T
@@ -232,8 +245,9 @@ where
         Self { inner, separator }
     }
 
+    /// Serialize the CSS Value with the specific serialization function.
     #[inline]
-    fn write_item<F>(&mut self, f: F) -> fmt::Result
+    pub fn write_item<F>(&mut self, f: F) -> fmt::Result
     where
         F: FnOnce(&mut CssWriter<'b, W>) -> fmt::Result,
     {
@@ -374,11 +388,11 @@ impl Separator for Space {
     where
         F: for<'tt> FnMut(&mut Parser<'i, 'tt>) -> Result<T, ParseError<'i, E>>,
     {
-        input.skip_whitespace(); // Unnecessary for correctness, but may help try() rewind less.
+        input.skip_whitespace(); // Unnecessary for correctness, but may help try_parse() rewind less.
         let mut results = vec![parse_one(input)?];
         loop {
-            input.skip_whitespace(); // Unnecessary for correctness, but may help try() rewind less.
-            if let Ok(item) = input.try(&mut parse_one) {
+            input.skip_whitespace(); // Unnecessary for correctness, but may help try_parse() rewind less.
+            if let Ok(item) = input.try_parse(&mut parse_one) {
                 results.push(item);
             } else {
                 return Ok(results);
@@ -399,14 +413,14 @@ impl Separator for CommaWithSpace {
     where
         F: for<'tt> FnMut(&mut Parser<'i, 'tt>) -> Result<T, ParseError<'i, E>>,
     {
-        input.skip_whitespace(); // Unnecessary for correctness, but may help try() rewind less.
+        input.skip_whitespace(); // Unnecessary for correctness, but may help try_parse() rewind less.
         let mut results = vec![parse_one(input)?];
         loop {
-            input.skip_whitespace(); // Unnecessary for correctness, but may help try() rewind less.
+            input.skip_whitespace(); // Unnecessary for correctness, but may help try_parse() rewind less.
             let comma_location = input.current_source_location();
-            let comma = input.try(|i| i.expect_comma()).is_ok();
-            input.skip_whitespace(); // Unnecessary for correctness, but may help try() rewind less.
-            if let Ok(item) = input.try(&mut parse_one) {
+            let comma = input.try_parse(|i| i.expect_comma()).is_ok();
+            input.skip_whitespace(); // Unnecessary for correctness, but may help try_parse() rewind less.
+            if let Ok(item) = input.try_parse(&mut parse_one) {
                 results.push(item);
             } else if comma {
                 return Err(comma_location.new_unexpected_token_error(Token::Comma));
@@ -497,66 +511,12 @@ macro_rules! impl_to_css_for_predefined_type {
 impl_to_css_for_predefined_type!(f32);
 impl_to_css_for_predefined_type!(i8);
 impl_to_css_for_predefined_type!(i32);
+impl_to_css_for_predefined_type!(u8);
 impl_to_css_for_predefined_type!(u16);
 impl_to_css_for_predefined_type!(u32);
 impl_to_css_for_predefined_type!(::cssparser::Token<'a>);
 impl_to_css_for_predefined_type!(::cssparser::UnicodeRange);
 
-/// Define an enum type with unit variants that each correspond to a CSS keyword.
-macro_rules! define_css_keyword_enum {
-    (pub enum $name:ident { $($variant:ident = $css:expr,)+ }) => {
-        #[allow(missing_docs)]
-        #[cfg_attr(feature = "servo", derive(Deserialize, Serialize))]
-        #[derive(Clone, Copy, Debug, Eq, Hash, MallocSizeOf, PartialEq, ToShmem)]
-        pub enum $name {
-            $($variant),+
-        }
-
-        impl $name {
-            /// Parse this property from a CSS input stream.
-            pub fn parse<'i, 't>(input: &mut ::cssparser::Parser<'i, 't>)
-                                 -> Result<$name, $crate::ParseError<'i>> {
-                use cssparser::Token;
-                let location = input.current_source_location();
-                match *input.next()? {
-                    Token::Ident(ref ident) => {
-                        Self::from_ident(ident).map_err(|()| {
-                            location.new_unexpected_token_error(
-                                Token::Ident(ident.clone()),
-                            )
-                        })
-                    }
-                    ref token => {
-                        Err(location.new_unexpected_token_error(token.clone()))
-                    }
-                }
-            }
-
-            /// Parse this property from an already-tokenized identifier.
-            pub fn from_ident(ident: &str) -> Result<$name, ()> {
-                match_ignore_ascii_case! { ident,
-                    $($css => Ok($name::$variant),)+
-                    _ => Err(())
-                }
-            }
-        }
-
-        impl $crate::ToCss for $name {
-            fn to_css<W>(
-                &self,
-                dest: &mut $crate::CssWriter<W>,
-            ) -> ::std::fmt::Result
-            where
-                W: ::std::fmt::Write,
-            {
-                match *self {
-                    $( $name::$variant => ::std::fmt::Write::write_str(dest, $css) ),+
-                }
-            }
-        }
-    };
-}
-
 /// Helper types for the handling of specified values.
 pub mod specified {
     use crate::ParsingMode;
