@@ -25,6 +25,8 @@
 // other warnings. PLEASE read the longer comment in
 // toolkit/components/reputationservice/ApplicationReputation.cpp
 // before modifying this list!
+// If you update this list, make sure to update the length of sExecutableExts
+// in nsLocalFileCommmon.h.
 /* static */
 const char* const sExecutableExts[] = {
     // clang-format off
@@ -35,10 +37,15 @@ const char* const sExecutableExts[] = {
   ".ad",
   ".ade",         // access project extension
   ".adp",
+  ".afploc",      // Apple Filing Protocol Location
   ".air",         // Adobe AIR installer
   ".app",         // executable application
   ".application", // from bug 348763
+  ".appref-ms",   // ClickOnce link
+  ".appx",
+  ".appxbundle",
   ".asp",
+  ".atloc",       // Appletalk Location
   ".bas",
   ".bat",
   ".cer",         // Signed certificate file
@@ -48,8 +55,10 @@ const char* const sExecutableExts[] = {
   ".cpl",
   ".crt",
   ".der",
+  ".diagcab",     // Windows archive
   ".exe",
   ".fileloc",     // Apple finder internet location data file
+  ".ftploc",      // Apple FTP Location
   ".fxp",         // FoxPro compiled app
   ".hlp",
   ".hta",
@@ -58,8 +67,12 @@ const char* const sExecutableExts[] = {
   ".ins",
   ".isp",
   ".jar",         // java application bundle
+#ifndef MOZ_ESR
+  ".jnlp",
+#endif
   ".js",
   ".jse",
+  ".library-ms",  // Windows Library Files
   ".lnk",
   ".mad",         // Access Module Shortcut
   ".maf",         // Access
@@ -86,6 +99,8 @@ const char* const sExecutableExts[] = {
   ".msh2xml",     // Microsoft Shell
   ".mshxml",      // Microsoft Shell
   ".msi",
+  ".msix",
+  ".msixbundle",
   ".msp",
   ".mst",
   ".ops",         // Office Profile Settings
@@ -102,6 +117,7 @@ const char* const sExecutableExts[] = {
   ".settingcontent-ms",
   ".shb",
   ".shs",
+  ".terminal",    // macOS terminal files
   ".url",
   ".vb",
   ".vbe",
@@ -124,7 +140,9 @@ const char* const sExecutableExts[] = {
   ".ws",
   ".wsc",
   ".wsf",
-  ".wsh"
+  ".wsh",
+  ".xll",         // MS Excel dynamic link library
+  ".xrm-ms"
     // clang-format on
 };
 
